@@ -39,11 +39,142 @@
 
 #include <QMap>
 
+#include <math.h>
+
+//----------------------------------------------------------
+
+#include "ColorConversions.h"
+//! @todo only include for TINY target
+
+const int HLSMAX = 255; //!< Used for computing tint
+
+//! @return tinted value for @a color
+//! Alpha value is left unchanged.
+/*! @param color to be converted
+    @param tint color tint: from -1.0 .. 1.0, where -1.0 means 100% darken
+           and 1.0 means 100% lighten; 0.0 means no change
+*/
+static QColor tintedColor(const QColor& color, qreal tint)
+{
+//kDebug() << "rgb:" << color.name() << "tint:" << tint;
+    if (tint == 0.0 || !color.isValid()) {
+        return color;
+    }
+    int h, l, s;
+    rgb_to_hls(color.red(), color.green(), color.blue(), &h, &l, &s);
+//    rgb_to_hls(0xec, 0xec, 0xec, &h, &l, &s);
+//kDebug() << "hls before:" << h << l << s;
+    if (tint < 0.0) {
+        l = floor( l * (1.0 + tint) );
+    }
+    else {
+        l = floor( l * (1.0 - tint) + (HLSMAX - HLSMAX * (1.0 - tint)) );
+    }
+//kDebug() << "hls after:" << h << l << s;
+    quint8 r, g, b;
+    hls_to_rgb(h, l, s, &r, &g, &b);
+//kDebug() << "rgb:" << r << g << b << QColor(r, g, b, color.alpha()).name();
+    return QColor(r, g, b, color.alpha());
+}
+
+/*! @return color decoded from a "rgb" attribute of the current element
+            or invalid QColor when reading was not possible.
+Used by:
+- color@rgb (§18.3.1.15)
+- fgColor@rgb (§18.8.19)
+- bgColor@rgb (§18.8.20) */
+static QColor readRgbAttribute(const QXmlStreamAttributes& attrs)
+{
+    TRY_READ_ATTR_WITHOUT_NS(rgb)
+//kDebug() << rgb;
+    return ST_UnsignedIntHex_to_QColor(rgb);
+}
+
+/*! @return tint value (-1.0..1.0) decoded from a "tint" attribute of the current element
+            or 0.0 when reading was not possible.
+Used by:
+- color@rgb (§18.3.1.15)
+- fgColor@rgb (§18.8.19)
+- bgColor@rgb (§18.8.20) */
+static qreal readTintAttribute(const QXmlStreamAttributes& attrs, const char* debugElement)
+{
+    TRY_READ_ATTR_WITHOUT_NS(tint)
+    qreal tintValue = 0.0;
+    STRING_TO_QREAL(tint, tintValue, QLatin1String(debugElement) + "@tint")
+kDebug() << tint << tintValue;
+    return tintValue;
+}
+
+//----------------------------------------------------------
+
 XlsxColorStyle::XlsxColorStyle()
-        : automatic(false), indexed(-1)
+        : automatic(false), indexed(-1), tint(0.0), theme(-1)
 {
 }
 
+bool XlsxColorStyle::isValid(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+{
+kDebug() << "indexed:" << indexed << "rgb:" << rgb.name() << "tint:" << tint << "theme:" << theme;
+    if (theme >= 0) {
+kDebug() << themeColor(themes).isValid();
+        return themeColor(themes).isValid();
+    }
+    return rgb.isValid();
+}
+
+QColor XlsxColorStyle::themeColor(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+{
+//! @todo find proper theme, not just any
+    MSOOXML::DrawingMLTheme *themeObject = themes->constBegin().value();
+kDebug() << themeObject;
+    if (themeObject) {
+        MSOOXML::DrawingMLColorSchemeItemBase *colorItemBase = themeObject->colorScheme.value(theme);
+kDebug() << colorItemBase;
+//        MSOOXML::DrawingMLColorSchemeItem* colorItem = colorItemBase ? colorItemBase->toColorItem() : 0;
+//kDebug() << colorItem;
+//        if (colorItem)
+        if (colorItemBase)
+            return colorItemBase->value();
+    }
+    return QColor();
+}
+
+QColor XlsxColorStyle::value(const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+{
+    QColor realColor;
+kDebug() << "theme:" << theme;
+    if (theme >= 0) {
+        realColor = themeColor(themes);
+kDebug() << "in theme found:" << realColor.name();
+    }
+    else {
+        realColor = rgb;
+kDebug() << "rgb found:" << realColor.name();
+    }
+
+    //themes->value(theme);
+    return tintedColor(realColor, tint);
+}
+
+KoFilter::ConversionStatus XlsxColorStyle::readAttributes(
+    const QXmlStreamAttributes& attrs, const char* debugElement)
+{
+    automatic = MSOOXML::Utils::convertBooleanAttr(attrs.value("auto").toString());
+    QString indexedStr;
+    TRY_READ_ATTR_WITHOUT_NS_INTO(indexed, indexedStr)
+    STRING_TO_INT(indexedStr, indexed, QLatin1String(debugElement) + "@indexed")
+//! @todo handle indexed
+    rgb = readRgbAttribute(attrs);
+    tint = readTintAttribute(attrs, debugElement);
+    QString themeStr;
+    TRY_READ_ATTR_WITHOUT_NS_INTO(theme, themeStr)
+    STRING_TO_INT(themeStr, theme, QLatin1String(debugElement) + "@theme")
+kDebug() << "indexed:" << indexed << "rgb:" << rgb.name() << "tint:" << tint << "theme:" << theme;
+    return KoFilter::OK;
+}
+
+//----------------------------------------------------------
+
 XlsxFontStyle::XlsxFontStyle()
         : underline(NoUnderline),
         bold(false),
@@ -55,54 +186,132 @@ XlsxFontStyle::XlsxFontStyle()
 }
 
 XlsxFillStyle::XlsxFillStyle()
-        : patternType(NonePatternType)
+        : patternType(NonePatternType),
+          cachedRealBackgroundColor( 0 )
 {
 }
 
-const XlsxColorStyle* XlsxFillStyle::realBackgroundColor() const
+XlsxFillStyle::~XlsxFillStyle()
 {
+    delete cachedRealBackgroundColor;
+}
+
+static QColor applyPatternDensity( const XlsxColorStyle& bg, const XlsxColorStyle& fg, qreal percent, const QMap<QString, MSOOXML::DrawingMLTheme*> *themes )
+{
+    const QColor bgColor = bg.theme >= 0 ? bg.themeColor( themes ) : bg.rgb.isValid() ? bg.rgb : QColor( Qt::white );
+    const QColor fgColor = fg.theme >= 0 ? fg.themeColor( themes ) : fg.rgb;//.isValid() ? fg.rgb : QColor( Qt::black );
+    
+    QColor result( Qt::white );
+    if( bgColor.isValid() ) {
+        result = QColor( bgColor.red() * percent, 
+                         bgColor.green() * percent, 
+                         bgColor.blue() * percent, 
+                         bgColor.alpha() );
+    }
+    if( fgColor.isValid() ) {
+        result = QColor( result.red()   + fgColor.red() * ( 1.0 - percent ), 
+                         result.green() + fgColor.green() * ( 1.0 - percent ), 
+                         result.blue()  + fgColor.blue() * ( 1.0 - percent ),
+                         bgColor.isValid() ? bgColor.alpha() : fgColor.alpha() );
+    }
+    return result;
+}
+
+const XlsxColorStyle* XlsxFillStyle::realBackgroundColor( const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
+{
+    delete cachedRealBackgroundColor;
+    cachedRealBackgroundColor = new XlsxColorStyle;
+
+kDebug() << "patternType:" << patternType;
     switch (patternType) {
     case NonePatternType:
         return 0;
     case SolidPatternType:
         return &fgColor;
-    default:;
-//! @todo support other patterns
+    case DarkDownPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.5, themes );
+        return cachedRealBackgroundColor;
+    case DarkGrayPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.25, themes );
+        return cachedRealBackgroundColor;
+    case DarkGridPatternType: // fall through
+    case DarkHorizontalPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.5, themes );
+        return cachedRealBackgroundColor;
+    case DarkTrellisPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.25, themes );
+        return cachedRealBackgroundColor;
+    case DarkUpPatternType:  // fall through
+    case DarkVerticalPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.50, themes );
+        return cachedRealBackgroundColor;
+    case LightPatternType:
+        break; //??
+    case LightDownPatternType:  // fall through
+    case LightGrayPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.75, themes );
+        return cachedRealBackgroundColor;
+    case LightGridPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.5625, themes );
+        return cachedRealBackgroundColor;
+    case LightHorizontalPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.75, themes );
+        return cachedRealBackgroundColor;
+    case LightTrellisPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.625, themes );
+        return cachedRealBackgroundColor;
+    case LightUpPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.75, themes );
+        return cachedRealBackgroundColor;
+    case LightVerticalPatternType: // fall through
+    case MediumGrayPatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.50, themes );
+        return cachedRealBackgroundColor;
+    case Gray0625PatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.9375, themes );
+        return cachedRealBackgroundColor;
+    case Gray125PatternType:
+        cachedRealBackgroundColor->rgb = applyPatternDensity( bgColor, fgColor, 0.875, themes );
+        return cachedRealBackgroundColor;
     }
     return &bgColor;
 }
 
-void XlsxFillStyle::setupCellStyle(KoGenStyle* cellStyle) const
+void XlsxFillStyle::setupCellStyle(KoGenStyle* cellStyle, const QMap<QString, MSOOXML::DrawingMLTheme*> *themes) const
 {
 //! @todo implement more styling;
-//!       use XlsxColorStyle::automatic, XlsxColorStyle::indexed, XlsxColorStyle::theme, XlsxColorStyle::tint...
-    const XlsxColorStyle* realBackgroundColor = this->realBackgroundColor();
+//!       use XlsxColorStyle::automatic, XlsxColorStyle::indexed, XlsxColorStyle::theme...
+    const XlsxColorStyle* realBackgroundColor = this->realBackgroundColor( themes );
     if (realBackgroundColor) {
-kDebug() << patternType << realBackgroundColor->rgb;
-        if (realBackgroundColor->rgb.isValid()) {
-            cellStyle->addProperty("fo:background-color", realBackgroundColor->rgb.name());
+kDebug() << patternType << realBackgroundColor->value(themes).name()
+         << realBackgroundColor->tint << realBackgroundColor->isValid(themes);
+        if (realBackgroundColor->isValid(themes)) {
+            cellStyle->addProperty("fo:background-color", realBackgroundColor->value(themes).name());
         }
     }
 }
 
+//----------------------------------------------------------
+
 class ST_UnderlineValue_fromStringMap : public QMap<QString, XlsxFontStyle::ST_UnderlineValue>
 {
 public:
     ST_UnderlineValue_fromStringMap() {
         insert(QLatin1String("single"), XlsxFontStyle::SingleUnderline);
-        insert(QLatin1String("double"), XlsxFontStyle::SingleUnderline);
+        insert(QLatin1String("double"), XlsxFontStyle::DoubleUnderline);
         insert(QLatin1String("singleAccounting"), XlsxFontStyle::SingleAccountingUnderline);
         insert(QLatin1String("doubleAccounting"), XlsxFontStyle::DoubleAccountingUnderline);
         insert(QLatin1String("none"), XlsxFontStyle::NoUnderline);
     }
 };
 
+// static
 XlsxFontStyle::ST_UnderlineValue XlsxFontStyle::ST_UnderlineValue_fromString(const QString& s)
 {
     K_GLOBAL_STATIC(ST_UnderlineValue_fromStringMap, s_ST_UnderlineValues)
-    kDebug() << s;
+//    kDebug() << s;
     const ST_UnderlineValue v = s_ST_UnderlineValues->value(s);
-    kDebug() << v;
+//    kDebug() << v;
     if (v == NoUnderline && s != "none")
         return SingleUnderline; // default
     return v;
@@ -111,11 +320,35 @@ XlsxFontStyle::ST_UnderlineValue XlsxFontStyle::ST_UnderlineValue_fromString(con
 void XlsxFontStyle::setUnderline(const QString& s)
 {
     underline = ST_UnderlineValue_fromString(s);
-    kDebug() << underline;
+//    kDebug() << underline;
+}
+
+ST_VerticalAlignRun::ST_VerticalAlignRun(const QString& msooxmlName)
+{
+    if (msooxmlName == QLatin1String("subscript"))
+        value = SubscriptVerticalAlignRun;
+    else if (msooxmlName == QLatin1String("superscript"))
+        value = SuperscriptVerticalAlignRun;
+    else
+        value = BaselineVerticalAlignRun;
+}
+
+void ST_VerticalAlignRun::setupCharacterStyle(KoCharacterStyle* characterStyle) const
+{
+    switch (value) {
+    case SubscriptVerticalAlignRun:
+        characterStyle->setVerticalAlignment(QTextCharFormat::AlignSubScript);
+        break;
+    case SuperscriptVerticalAlignRun:
+        characterStyle->setVerticalAlignment(QTextCharFormat::AlignSuperScript);
+        break;
+    default:;
+    }
 }
 
 void XlsxFontStyle::setupCharacterStyle(KoCharacterStyle* characterStyle) const
 {
+    // line
     switch (underline) {
     case SingleUnderline:
     case DoubleUnderline:
@@ -127,6 +360,7 @@ void XlsxFontStyle::setupCharacterStyle(KoCharacterStyle* characterStyle) const
     default:;
     }
 
+    // # of lines
     switch (underline) {
     case SingleUnderline:
     case SingleAccountingUnderline:
@@ -162,9 +396,9 @@ void XlsxFontStyle::setupCharacterStyle(KoCharacterStyle* characterStyle) const
         qreal width;
         characterStyle->underlineWidth(weight, width);
 
-        kDebug() << "underlineStyle:" << characterStyle->underlineStyle()
+/*        kDebug() << "underlineStyle:" << characterStyle->underlineStyle()
         << "underlineType:" << characterStyle->underlineType()
-        << "underlineWeight:" << weight;
+        << "underlineWeight:" << weight;*/
     }
 
     if (bold)
@@ -173,21 +407,59 @@ void XlsxFontStyle::setupCharacterStyle(KoCharacterStyle* characterStyle) const
         characterStyle->setFontItalic(true);
     if (!m_defaultSize)
         characterStyle->setFontPointSize(m_size);
+
+    vertAlign.setupCharacterStyle(characterStyle);
 }
 
-void XlsxFontStyle::setupCellTextStyle(KoGenStyle* cellStyle) const
+void XlsxFontStyle::setupCellTextStyle(
+    const QMap<QString, MSOOXML::DrawingMLTheme*> *themes,
+    KoGenStyle* cellStyle) const
 {
     if (!name.isEmpty()) {
-        cellStyle->addProperty("style:font-name", name, KoGenStyle::TextType);
+#ifdef __GNUC__
+#warning TODO: we are saving with fo:font-family now because style:font-name is not properly supported by kotext; fix void KoCharacterStyle::loadOdf(KoOdfLoadingContext &context)...
+#endif
+//!@ todo reenable this        cellStyle->addProperty("style:font-name", name, KoGenStyle::TextType);
+        cellStyle->addProperty("fo:font-family", name, KoGenStyle::TextType);
     }
-    if (color.rgb.isValid()) {
-        cellStyle->addProperty("fo:color", color.rgb.name(), KoGenStyle::TextType);
+    if (color.isValid(themes)) {
+        const QColor c(color.value(themes));
+        cellStyle->addProperty("fo:color", c.name(), KoGenStyle::TextType);
     }
     //! @todo implement more styling
 }
 
 XlsxStyles::XlsxStyles()
 {
+    // fill the default number formats
+    // from Office Open XML Part 4 - Markup Language Reference, p. 1974
+    numberFormatStrings[ 1 ] = QLatin1String( "0" );
+    numberFormatStrings[ 2 ] = QLatin1String( "0.00" );
+    numberFormatStrings[ 3 ] = QLatin1String( "#,##0" );
+    numberFormatStrings[ 4 ] = QLatin1String( "#,##0.00" );
+    numberFormatStrings[ 9 ] = QLatin1String( "0%" );
+    numberFormatStrings[ 10 ] = QLatin1String( "0.00%" );
+    numberFormatStrings[ 11 ] = QLatin1String( "0.00E+00" );
+    numberFormatStrings[ 12 ] = QLatin1String( "# ?/?" );
+    numberFormatStrings[ 13 ] = QLatin1String( "# \?\?/\?\?" );
+    numberFormatStrings[ 14 ] = QLatin1String( "mm-dd-yy" );
+    numberFormatStrings[ 15 ] = QLatin1String( "d-mmm-yy" );
+    numberFormatStrings[ 16 ] = QLatin1String( "d-mmm" );
+    numberFormatStrings[ 17 ] = QLatin1String( "mmm-yy" );
+    numberFormatStrings[ 18 ] = QLatin1String( "h:mm AM/PM" );
+    numberFormatStrings[ 19 ] = QLatin1String( "h:mm:ss AM/PM" );
+    numberFormatStrings[ 20 ] = QLatin1String( "h:mm" );
+    numberFormatStrings[ 21 ] = QLatin1String( "h:mm:ss" );
+    numberFormatStrings[ 22 ] = QLatin1String( "m/d/yy h:mm" );
+    numberFormatStrings[ 37 ] = QLatin1String( "#,##0 ;(#,##0)" );
+    numberFormatStrings[ 38 ] = QLatin1String( "#,##0 ;[Red](#,##0)" );
+    numberFormatStrings[ 39 ] = QLatin1String( "#,##0.00;(#,##0.00)" );
+    numberFormatStrings[ 40 ] = QLatin1String( "#,##0.00;[Red](#,##0.00)" );
+    numberFormatStrings[ 45 ] = QLatin1String( "mm:ss" );
+    numberFormatStrings[ 46 ] = QLatin1String( "[h]:mm:ss" );
+    numberFormatStrings[ 47 ] = QLatin1String( "mmss.0" );
+    numberFormatStrings[ 48 ] = QLatin1String( "##0.0E+0" );
+    numberFormatStrings[ 49 ] = QLatin1String( "@" );
 }
 
 XlsxStyles::~XlsxStyles()
@@ -215,7 +487,10 @@ XlsxCellFormat::XlsxCellFormat()
    borderId(-1), fillId(-1), fontId(-1), numFmtId(-1),
    pivotButton(false), quotePrefix(false), xfId(-1),
    horizontalAlignment(GeneralHorizontalAlignment),
-   verticalAlignment(NoVerticalAlignment)
+   verticalAlignment(NoVerticalAlignment),
+   wrapText(false),
+   shrinkToFit(false),
+   textRotation(0)
 {
 }
 
@@ -227,7 +502,7 @@ bool XlsxCellFormat::setupCharacterStyle(const XlsxStyles *styles, KoCharacterSt
 {
     XlsxFontStyle* fontStyle = styles->fontStyle(fontId);
     if (!fontStyle) {
-        kDebug() << "No font with ID:" << fontId;
+        kWarning() << "No font with ID:" << fontId;
         return false;
     }
     fontStyle->setupCharacterStyle(characterStyle);
@@ -238,6 +513,7 @@ class ST_HorizontalAlignment_fromStringMap : public QMap<QString, XlsxCellFormat
 {
 public:
     ST_HorizontalAlignment_fromStringMap() {
+        insert(QString(), XlsxCellFormat::GeneralHorizontalAlignment);
         insert(QLatin1String("general"), XlsxCellFormat::GeneralHorizontalAlignment);
         insert(QLatin1String("center"), XlsxCellFormat::CenterHorizontalAlignment);
         insert(QLatin1String("centerContinuous"), XlsxCellFormat::CenterContinuousHorizontalAlignment);
@@ -259,7 +535,7 @@ class ST_VerticalAlignment_fromStringMap : public QMap<QString, XlsxCellFormat::
 {
 public:
     ST_VerticalAlignment_fromStringMap() {
-        insert(QLatin1String(""), XlsxCellFormat::NoVerticalAlignment);
+        insert(QString(), XlsxCellFormat::NoVerticalAlignment);
         insert(QLatin1String("bottom"), XlsxCellFormat::BottomVerticalAlignment);
         insert(QLatin1String("center"), XlsxCellFormat::CenterVerticalAlignment);
         insert(QLatin1String("distributed"), XlsxCellFormat::DistributedVerticalAlignment);
@@ -278,16 +554,32 @@ void XlsxCellFormat::setVerticalAlignment(const QString& alignment)
 void XlsxCellFormat::setupCellStyleAlignment(KoGenStyle* cellStyle) const
 {
 //! @todo FillHorizontalAlignment, JustifyHorizontalAlignment
-    bool wrapOption = false;
+    int wrapOption = -1; // "don't know"
+    if (wrapText)
+        wrapOption = 1;
+    //special case: 255 indicates vertical rotation without rotated characters (I couldn't find that documented though)
+    const bool verticalTtb = textRotation == 255;
+    if (verticalTtb)
+        cellStyle->addProperty("style:direction", "ttb");
+    else if (textRotation != 0) {
+        //@todo map other cases (to style:text-rotate-angle? that one only allows 0, 90, 270)
+    }
+
+    if (shrinkToFit)
+        cellStyle->addProperty("style:shrink-to-fit", "true");
+
     switch (horizontalAlignment) {
     case CenterHorizontalAlignment:
     case CenterContinuousHorizontalAlignment:
     case DistributedHorizontalAlignment:
         cellStyle->addProperty("fo:text-align", "center", KoGenStyle::ParagraphType);
         if (horizontalAlignment == DistributedHorizontalAlignment)
-            wrapOption = true;
+            wrapOption = 1;
         break;
     case GeneralHorizontalAlignment: // ok?
+        if (verticalTtb) // Excel centers vertical text by default, so mimic that
+            cellStyle->addProperty("fo:text-align", "center", KoGenStyle::ParagraphType);
+        break;
     case LeftHorizontalAlignment:
         cellStyle->addProperty("fo:text-align", "start", KoGenStyle::ParagraphType);
         break;
@@ -309,21 +601,27 @@ void XlsxCellFormat::setupCellStyleAlignment(KoGenStyle* cellStyle) const
         cellStyle->addProperty("style:vertical-align", "top");
         break;
     case NoVerticalAlignment:
-    case JustifyVerticalAlignment: // ok?
+    case JustifyVerticalAlignment: // ok?        
     case DistributedVerticalAlignment:
+        cellStyle->addProperty("style:vertical-align", "top");
+        cellStyle->addProperty("koffice:vertical-distributed", "distributed");
+        wrapOption = 1;
+        break;
     case BottomVerticalAlignment:
-        if (verticalAlignment == DistributedVerticalAlignment)
-            wrapOption = true;
     default:;
     }
 
-//! @todo take alignment/@wrapText into account
-    cellStyle->addProperty("fo:wrap-option", wrapOption ? "wrap" : "no-wrap");
+    if (wrapOption == 0 || wrapOption == 1)
+        cellStyle->addProperty("fo:wrap-option", wrapOption ? "wrap" : "no-wrap");
 }
 
 //! See http://www.w3.org/TR/2001/REC-xsl-20011015/slice7.html#text-align
-bool XlsxCellFormat::setupCellStyle(const XlsxStyles *styles, KoGenStyle* cellStyle) const
+bool XlsxCellFormat::setupCellStyle(
+    const XlsxStyles *styles,
+    const QMap<QString, MSOOXML::DrawingMLTheme*> *themes,
+    KoGenStyle* cellStyle) const
 {
+    kDebug() << "fontId:" << fontId << "fillId:" << fillId;
     if (applyAlignment) {
         setupCellStyleAlignment(cellStyle);
     }
@@ -333,16 +631,15 @@ bool XlsxCellFormat::setupCellStyle(const XlsxStyles *styles, KoGenStyle* cellSt
             kWarning() << "No font with ID:" << fontId;
             return false;
         }
-        fontStyle->setupCellTextStyle(cellStyle);
+        fontStyle->setupCellTextStyle(themes, cellStyle);
     }
     if (applyFill && fillId >= 0) {
-kDebug() << "fillId:" << fillId;
         XlsxFillStyle *fillStyle = styles->fillStyle(fillId);
         if (!fillStyle) {
             kWarning() << "No fill with ID:" << fillId;
             return false;
         }
-        fillStyle->setupCellStyle(cellStyle);
+        fillStyle->setupCellStyle(cellStyle, themes);
     }
     return true;
 }
@@ -454,7 +751,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::readInternal()
  - extLst (Future Feature Data Storage Area) §18.2.10
  - [done] fills (Fills) §18.8.21
  - [done] fonts (Fonts) §18.8.23
- - numFmts (Number Formats) §18.8.31
+ - [done] numFmts (Number Formats) §18.8.31
  - tableStyles (Table Styles) §18.8.42
 */
 KoFilter::ConversionStatus XlsxXmlStylesReader::read_styleSheet()
@@ -466,8 +763,9 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_styleSheet()
         kDebug() << *this;
         if (isStartElement()) {
             TRY_READ_IF(fonts)
-            TRY_READ_IF(fills)
-            TRY_READ_IF(cellXfs)
+            ELSE_TRY_READ_IF(fills)
+            ELSE_TRY_READ_IF(numFmts)
+            ELSE_TRY_READ_IF(cellXfs)
 //! @todo add ELSE_WRONG_FORMAT
         }
         BREAK_IF_END_OF(CURRENT_EL);
@@ -490,11 +788,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fonts()
 {
     READ_PROLOGUE
     if (!m_context->styles->fontStyles.isEmpty()) {
-        ERROR_UNEXPECTED_SECOND_OCCURENCE(CURRENT_EL);
+        raiseUnexpectedSecondOccurenceOfElError(STRINGIFY(CURRENT_EL));
     }
     const QXmlStreamAttributes attrs(attributes());
     TRY_READ_ATTR_WITHOUT_NS(count)
-    uint countNumber;
+    uint countNumber = 0;
     STRING_TO_INT(count, countNumber, "styleSheet/fonts@count")
     m_context->styles->fontStyles.resize(countNumber);
     m_fontStyleIndex = 0;
@@ -511,6 +809,80 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fonts()
     READ_EPILOGUE
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL numFmts
+//! numFmts handler (Number formats)
+/*! ECMA-376, 18.8.31, p. 1981.
+ This element defines the number formats in this workbook, consisting of a sequence of numFmt records,
+ where each numFmt record defines a particular number format, indicating how to format and render
+ the numeric value of a cell.
+
+ Child elements:
+ - [done] numFmt (Format Definition) §18.8.30
+ Parent elements:
+ - [done] styleSheet (§18.8.39)
+*/
+KoFilter::ConversionStatus XlsxXmlStylesReader::read_numFmts()
+{
+    READ_PROLOGUE
+    const QXmlStreamAttributes attrs(attributes());
+    TRY_READ_ATTR_WITHOUT_NS( count )
+    int countNumber = 0;
+    STRING_TO_INT( count, countNumber, "styleSheet/numFmts@count" );
+
+    while( !atEnd() )
+    {
+        readNext();
+        if( isStartElement() )
+        {
+            TRY_READ_IF( numFmt )
+            ELSE_WRONG_FORMAT
+        }
+        BREAK_IF_END_OF( CURRENT_EL )
+    }
+
+    READ_EPILOGUE
+}
+
+#undef CURRENT_EL
+#define CURRENT_EL numFmt
+//! numFmt handler (Number format)
+/*! ECMA-376, 18.8.30, p. 1973.
+ This element specifies number format properties which indicate how to format and render
+ the numeric value of a cell.
+
+ No child elements.
+ 
+ Parent elements:
+ - dxf (§18.8.14)
+ - ndxf (§18.11.1.4)
+ - [done] numFmts (§18.8.31)
+ - odxf (§18.11.1.6)
+*/
+KoFilter::ConversionStatus XlsxXmlStylesReader::read_numFmt()
+{
+    READ_PROLOGUE
+
+    const QXmlStreamAttributes attrs(attributes());
+    TRY_READ_ATTR_WITHOUT_NS( numFmtId )
+    int id = -1;
+    STRING_TO_INT(numFmtId, id, "numFmt@numFmtId")
+
+    TRY_READ_ATTR_WITHOUT_NS( formatCode );
+    if (id >= 0 && !formatCode.isEmpty()) {
+        m_context->styles->numberFormatStrings[ id ] = formatCode;
+    }
+
+    while( true )
+    {
+        BREAK_IF_END_OF( CURRENT_EL );
+        readNext();
+        break;
+    }
+
+    READ_EPILOGUE
+}
+
 #undef CURRENT_EL
 #define CURRENT_EL font
 //! font handler (Font)
@@ -565,6 +937,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_font()
             ELSE_TRY_READ_IF(strike)
             ELSE_TRY_READ_IF(u)
             ELSE_TRY_READ_IF(color)
+            ELSE_TRY_READ_IF(vertAlign)
 //! @todo add ELSE_WRONG_FORMAT
         }
         BREAK_IF_END_OF(CURRENT_EL);
@@ -615,6 +988,31 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_sz()
     READ_EPILOGUE
 }
 
+#undef CURRENT_EL
+#define CURRENT_EL vertAlign
+//! vertAlign handler (Vertical Alignment)
+/*! ECMA-376, ???
+
+ No child elements.
+ Parent elements:
+ - [done] font (§18.8.22)
+*/
+KoFilter::ConversionStatus XlsxXmlStylesReader::read_vertAlign()
+{
+    Q_ASSERT(m_currentFontStyle);
+
+    READ_PROLOGUE
+    const QXmlStreamAttributes attrs(attributes());
+    TRY_READ_ATTR_WITHOUT_NS(val)
+    m_currentFontStyle->vertAlign = ST_VerticalAlignRun( val );
+
+    while (true) {
+        BREAK_IF_END_OF(CURRENT_EL);
+        readNext();
+        break;
+    }
+    READ_EPILOGUE
+}
 #undef CURRENT_EL
 #define CURRENT_EL name
 //! name handler (Font Name)
@@ -780,7 +1178,8 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_color()
     Q_ASSERT(m_currentFontStyle);
 
     READ_PROLOGUE
-    RETURN_IF_ERROR( readColorStyle(&m_currentFontStyle->color, "color") )
+    const QXmlStreamAttributes attrs(attributes());
+    RETURN_IF_ERROR( m_currentFontStyle->color.readAttributes(attrs, "color") )
     while (true) {
         BREAK_IF_END_OF(CURRENT_EL);
         readNext();
@@ -800,16 +1199,21 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_color()
  - [done] xf (Format) §18.8.45
  Parent elements:
  - [done] styleSheet (§18.8.39)
+
+ @note cellStyleXfs: the standard states that both the cell style xf records and cell xf records
+       must be read to understand the full set of formatting applied to a cell.
+       In MSO, only the cell xf record defines the formatting applied to a cell.
+       See <a href="http://www.documentinteropinitiative.org/implnotes/implementationnotelist.aspx?id=dd2615fe-aa8d-4a06-a415-13389919cf36&specname=ecma-376">here</a>.
 */
 KoFilter::ConversionStatus XlsxXmlStylesReader::read_cellXfs()
 {
     READ_PROLOGUE
     if (!m_context->styles->cellFormats.isEmpty()) {
-        ERROR_UNEXPECTED_SECOND_OCCURENCE(CURRENT_EL);
+        raiseUnexpectedSecondOccurenceOfElError(STRINGIFY(CURRENT_EL));
     }
     const QXmlStreamAttributes attrs(attributes());
     TRY_READ_ATTR_WITHOUT_NS(count)
-    uint countNumber;
+    uint countNumber = 0;
     STRING_TO_INT(count, countNumber, "styleSheet/cellXfs@count")
     m_context->styles->cellFormats.resize(countNumber);
     m_cellFormatIndex = 0;
@@ -883,7 +1287,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_xf()
     STRING_TO_INT(borderId, m_currentCellFormat->borderId, "xf@borderId")
 
     TRY_READ_ATTR_WITHOUT_NS(fillId)
-kDebug() << "fillId:" << fillId;
     STRING_TO_INT(fillId, m_currentCellFormat->fillId, "xf@fillId")
 
     TRY_READ_ATTR_WITHOUT_NS(fontId)
@@ -941,6 +1344,13 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_alignment()
     kDebug() << "horizontalAlignment:" << m_currentCellFormat->horizontalAlignment;
     m_currentCellFormat->setVerticalAlignment(attrs.value("vertical").toString());
     kDebug() << "verticalAlignment:" << m_currentCellFormat->verticalAlignment;
+    const bool wrap = readBooleanAttr("wrapText", false);
+    m_currentCellFormat->wrapText = wrap;
+    const bool shrinkToFit = readBooleanAttr("shrinkToFit", false);
+    m_currentCellFormat->shrinkToFit = shrinkToFit;
+    const uint textRotation = attributes().value("textRotation").toString().toUInt();
+    m_currentCellFormat->textRotation = textRotation;
+
 //! @todo more attributes
 
     while (true) {
@@ -967,11 +1377,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fills()
 {
     READ_PROLOGUE
     if (!m_context->styles->fillStyles.isEmpty()) {
-        ERROR_UNEXPECTED_SECOND_OCCURENCE(CURRENT_EL);
+        raiseUnexpectedSecondOccurenceOfElError(STRINGIFY(CURRENT_EL));
     }
     const QXmlStreamAttributes attrs(attributes());
     TRY_READ_ATTR_WITHOUT_NS(count)
-    uint countNumber;
+    uint countNumber = 0;
     STRING_TO_INT(count, countNumber, "styleSheet/fills@count")
     m_context->styles->fillStyles.resize(countNumber);
     m_fillStyleIndex = 0;
@@ -1039,7 +1449,7 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_fill()
 void XlsxXmlStylesReader::handlePatternType(const QString& patternType)
 {
     const QByteArray p(patternType.toLatin1());
-kDebug() << p;
+//kDebug() << p;
     if (p.isEmpty() || p == MsooXmlReader::constNone) {
         // 100% background
         m_currentFillStyle->patternType = XlsxFillStyle::NonePatternType;
@@ -1050,41 +1460,58 @@ kDebug() << p;
     }
     else if (p.startsWith("dark")) {
         if (p == "darkDown") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkDownPatternType;
         }
         else if (p == "darkGray") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkGrayPatternType;
         }
         else if (p == "darkGrid") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkGridPatternType;
         }
         else if (p == "darkHorizontal") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkGridPatternType;
         }
         else if (p == "darkTrellis") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkTrellisPatternType;
         }
         else if (p == "darkUp") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkUpPatternType;
         }
         else if (p == "darkVertical") {
+            m_currentFillStyle->patternType = XlsxFillStyle::DarkVerticalPatternType;
         }
     }
     else if (p.startsWith("light")) {
         if (p == "lightDown") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightDownPatternType;
         }
         else if (p == "lightGray") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightGrayPatternType;
         }
         else if (p == "lightGrid") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightGridPatternType;
         }
         else if (p == "lightHorizontal") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightHorizontalPatternType;
         }
         else if (p == "lightTrellis") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightTrellisPatternType;
         }
         else if (p == "lightUp") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightUpPatternType;
         }
         else if (p == "lightVertical") {
+            m_currentFillStyle->patternType = XlsxFillStyle::LightVerticalPatternType;
         }
     }
     else if (p == "mediumGray") {
+        m_currentFillStyle->patternType = XlsxFillStyle::MediumGrayPatternType;
     }
     else if (p == "gray0625") {
+        m_currentFillStyle->patternType = XlsxFillStyle::Gray0625PatternType;
     }
     else if (p == "gray125") {
+        m_currentFillStyle->patternType = XlsxFillStyle::Gray125PatternType;
     }
     else {
         kWarning() << "unknown value" << p
@@ -1126,28 +1553,6 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_patternFill()
     READ_EPILOGUE
 }
 
-// private
-QColor XlsxXmlStylesReader::readRgbAttribute(const QXmlStreamAttributes& attrs) const
-{
-    TRY_READ_ATTR_WITHOUT_NS(rgb)
-    if (rgb.isEmpty())
-        return QColor();
-    return ST_UnsignedIntHex_to_QColor(rgb);
-}
-
-KoFilter::ConversionStatus XlsxXmlStylesReader::readColorStyle(
-    XlsxColorStyle* colorStyle, const char* debugElement)
-{
-    const QXmlStreamAttributes attrs(attributes());
-    colorStyle->automatic = readBooleanAttr("auto", false);
-    TRY_READ_ATTR_WITHOUT_NS(indexed)
-    STRING_TO_INT(indexed, colorStyle->indexed, QLatin1String(debugElement) + "@indexed")
-    colorStyle->rgb = readRgbAttribute(attrs);
- //! @todo theme
- //! @todo tint
-    return KoFilter::OK;
-}
-
 #undef CURRENT_EL
 #define CURRENT_EL bgColor
 //! bgColor handler (Background Color)
@@ -1165,7 +1570,9 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_bgColor()
 {
     Q_ASSERT(m_currentFillStyle);
     READ_PROLOGUE
-    RETURN_IF_ERROR( readColorStyle(&m_currentFillStyle->bgColor, "bgColor") )
+
+    const QXmlStreamAttributes attrs(attributes());
+    RETURN_IF_ERROR( m_currentFillStyle->bgColor.readAttributes(attrs, "bgColor") )
 
     while (true) {
         BREAK_IF_END_OF(CURRENT_EL);
@@ -1190,8 +1597,11 @@ KoFilter::ConversionStatus XlsxXmlStylesReader::read_bgColor()
 */
 KoFilter::ConversionStatus XlsxXmlStylesReader::read_fgColor()
 {
+    Q_ASSERT(m_currentFillStyle);
     READ_PROLOGUE
-    RETURN_IF_ERROR( readColorStyle(&m_currentFillStyle->fgColor, "fgColor") )
+
+    const QXmlStreamAttributes attrs(attributes());
+    RETURN_IF_ERROR( m_currentFillStyle->fgColor.readAttributes(attrs, "fgColor") )
 
     while (true) {
         BREAK_IF_END_OF(CURRENT_EL);
