@@ -42,6 +42,8 @@
 #include <KoGenStyle.h>
 #include <KoFontFace.h>
 
+#include "document.h"
+
 wvWare::U8 KWordReplacementHandler::hardLineBreak()
 {
     return '\n';
@@ -57,30 +59,32 @@ wvWare::U8 KWordReplacementHandler::nonRequiredHyphen()
     return 0xad; // soft hyphen, according to kword.dtd
 }
 
-
 KWordTextHandler::KWordTextHandler(wvWare::SharedPtr<wvWare::Parser> parser, KoXmlWriter* bodyWriter, KoGenStyles* mainStyles)
-        : m_writingHeader(false)
-        , m_writeMasterStyleName(false)
-        , m_headerWriter(0)
-        , m_mainStyles(0)
-        , m_sectionNumber(0)
-        , m_document(0)
-        , m_parser(parser)
-        , m_footNoteNumber(0)
-        , m_endNoteNumber(0)
-        , m_index(0)
-        , m_currentTable(0)
-        , m_paragraph(0)
-        , m_insideField(false)
-        , m_fieldAfterSeparator(false)
-        , m_fieldType(0)
-        , m_insideFootnote(false)
-        , m_footnoteWriter(0)
-        , m_footnoteBuffer(0)
-        , m_maxColumns(0)
-        , m_currentListDepth(-1)
-        , m_currentListID(0)
-        , m_previousListID(0)
+    : m_mainStyles(0)
+    , m_sectionNumber(0)
+    , m_document(0)
+    , m_parser(parser)
+    , m_footNoteNumber(0)
+    , m_endNoteNumber(0)
+    , m_index(0)
+    , m_currentTable(0)
+    , m_paragraph(0)
+    , m_insideField(false)
+    , m_fieldAfterSeparator(false)
+    , m_fieldType(0)
+    , m_insideFootnote(false)
+    , m_footnoteWriter(0)
+    , m_footnoteBuffer(0)
+    , m_insideAnnotation(false)
+    , m_annotationWriter(0)
+    , m_annotationBuffer(0)
+    , m_insideDrawing(false)
+    , m_drawingWriter(0)
+    , m_maxColumns(0)
+    , m_currentListDepth(-1)
+    , m_currentListID(0)
+    , m_previousListID(0)
+    , m_hyperLinkActive(false)
 {
 #ifdef IMAGE_IMPORT
     kDebug(30513) << "we have image support";
@@ -97,6 +101,11 @@ KWordTextHandler::KWordTextHandler(wvWare::SharedPtr<wvWare::Parser> parser, KoX
     } else {
         kWarning() << "No mainStyles!";
     }
+
+    //[MS-DOC] — v20090708 - 2.7.2 DopBase pg.163
+    if ((0x00D9 >= m_parser->fib().nFib) && (m_parser->dop().nfcFtnRef2 == 0)) {
+        m_footNoteNumber = m_parser->dop().nFtn - 1;
+    }
 }
 
 //increment m_sectionNumber
@@ -109,7 +118,7 @@ void KWordTextHandler::sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP>
     //store sep for section end
     m_sep = sep;
 
-    //sep->bkc 0=no break, 1=new column, 2=new page, 3=even page, 4=odd page
+    //break code
     kDebug(30513) << "sep->bkc = " << sep->bkc;
 
     int numColumns = sep->ccolM1 + 1;
@@ -121,13 +130,12 @@ void KWordTextHandler::sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP>
     //check for a column break
     if (sep->bkc == 1) {
     }
-    //check for change in number of columns, too
-    //if sep->bkc isn't 0, just check to see if we have
-    // more than the usual one column
+    //check for change in number of columns, too if sep->bkc isn't 0, just
+    //check to see if we have more than the usual one column
     if (numColumns > 1 || sep->bkc == 0) {
         QString sectionStyleName = "Sect";
         sectionStyleName.append(QString::number(m_sectionNumber));
-        KoGenStyle sectionStyle(KoGenStyle::StyleSectionAuto, "section");
+        KoGenStyle sectionStyle(KoGenStyle::SectionAutoStyle, "section");
         //parse column info
         QBuffer buf;
         buf.open(QIODevice::WriteOnly);
@@ -164,8 +172,8 @@ void KWordTextHandler::sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP>
                 sectionStyle.addChildElement("text:notes-configuration", footconfig);
         */
         //add style to the collection
-        sectionStyleName = m_mainStyles->lookup(sectionStyle, sectionStyleName,
-                                                KoGenStyles::DontForceNumbering);
+        sectionStyleName = m_mainStyles->insert(sectionStyle, sectionStyleName,
+                                                KoGenStyles::DontAddNumberToName);
         //put necessary tags in the content
         m_bodyWriter->startElement("text:section");
         QString sectionName = "Section";
@@ -178,7 +186,7 @@ void KWordTextHandler::sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP>
 void KWordTextHandler::sectionEnd()
 {
     kDebug(30513);
-    if (m_sep->bkc >= 2) {
+    if (m_sep->bkc != 1) {
         emit sectionEnd(m_sep);
     }
     if (m_sep->ccolM1 > 0 || m_sep->bkc == 0) {
@@ -193,14 +201,15 @@ void KWordTextHandler::headersFound(const wvWare::HeaderFunctor& parseHeaders)
     //only parse headers if we're in a section that can have new headers
     //ie. new sections for columns trigger this function again, but we've
     //already parsed the headers
-    if (m_sep->bkc >= 2) {
+    if (m_sep->bkc != 1) {
         emit headersFound(new wvWare::HeaderFunctor(parseHeaders), 0);
     }
 }
 
+
 //this part puts the marker in the text, and signals for the rest to be parsed later
 void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
-                                     wvWare::UChar character, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                                     wvWare::UString characters, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                      const wvWare::FootnoteFunctor& parseFootnote)
 {
     Q_UNUSED(chp);
@@ -209,7 +218,7 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
 
     m_insideFootnote = true;
 
-    //create temp writer for footnote contentthat we'll add to m_paragraph
+    //create temp writer for footnote content that we'll add to m_paragraph
     m_footnoteBuffer = new QBuffer();
     m_footnoteBuffer->open(QIODevice::WriteOnly);
     m_footnoteWriter = new KoXmlWriter(m_footnoteBuffer);
@@ -219,7 +228,7 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
     m_footnoteWriter->addAttribute("text:note-class", type == wvWare::FootnoteData::Endnote ? "endnote" : "footnote");
     //autonumber or character
     m_footnoteWriter->startElement("text:note-citation");
-    if (character.unicode() == 2) {//autonumbering: 1,2,3,... for footnote; i,ii,iii,... for endnote
+    if (characters[0].unicode() == 2) {//autonumbering: 1,2,3,... for footnote; i,ii,iii,... for endnote
         //Note: besides converting the number to text here the format is specified in section-properties -> notes-configuration too
 
         int noteNumber = (type == wvWare::FootnoteData::Endnote ? ++m_endNoteNumber : ++m_footNoteNumber);
@@ -232,30 +241,41 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
             break;
         case 1: // uppercase roman
         case 2:  { // lowercase roman
-            QString numDigitsLower[] = {"m", "cm", "d", "cd", "c", "xc", "l", "xl", "x", "ix", "v", "iv", "i" };
-            QString numDigitsUpper[] = {"M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX", "V", "IV", "I" };
-            QString *numDigits = (m_parser->dop().nfcFtnRef2 == 1 ? numDigitsUpper : numDigitsLower);
-            int numValues[] = {1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1};
+                QString numDigitsLower[] = {"m", "cm", "d", "cd", "c", "xc", "l", "xl", "x", "ix", "v", "iv", "i" };
+                QString numDigitsUpper[] = {"M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX", "V", "IV", "I" };
+                QString *numDigits = (m_parser->dop().nfcFtnRef2 == 1 ? numDigitsUpper : numDigitsLower);
+                int numValues[] = {1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1};
 
-            for (int i = 0; i < 13; ++i) {
-                while (noteNumber >= numValues[i]) {
-                    noteNumber -= numValues[i];
-                    noteNumberString += numDigits[i];
+                for (int i = 0; i < 13; ++i) {
+                    while (noteNumber >= numValues[i]) {
+                        noteNumber -= numValues[i];
+                        noteNumberString += numDigits[i];
+                    }
                 }
+                break;
             }
-            break;
-        }
         case 3: // uppercase letter
             letter = 'A';
         case 4: { // lowercase letter
-            while (noteNumber / 25 > 0) {
-                noteNumberString += QString::number(noteNumber / 25);
-                noteNumber = noteNumber % 25;
-                noteNumberString += QChar(letter - 1 + noteNumber / 25);
+                while (noteNumber / 25 > 0) {
+                    noteNumberString += QString::number(noteNumber / 25);
+                    noteNumber = noteNumber % 25;
+                    noteNumberString += QChar(letter - 1 + noteNumber / 25);
+                }
+                noteNumberString += QChar(letter - 1 + noteNumber);
+                break;
+            }
+        case 9: {
+                QChar chicagoStyle[] =  {42, 8224, 8225, 167};
+                int styleIndex = (noteNumber - 1) % 4;
+                int repeatCount = (noteNumber - 1) / 4;
+                noteNumberString = QString(chicagoStyle[styleIndex]);
+                while (repeatCount > 0) {
+                    noteNumberString += QString(chicagoStyle[styleIndex]);
+                    repeatCount--;
+                }
+                break;
             }
-            noteNumberString += QChar(letter - 1 + noteNumber);
-            break;
-        }
         default:
             noteNumberString = QString::number(noteNumber);
             break;
@@ -263,7 +283,14 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
 
         m_footnoteWriter->addTextNode(noteNumberString);
     } else {
-        m_footnoteWriter->addTextNode(QString(QChar(character.unicode())));
+        int index = 0;
+        int length = characters.length();
+        QString customNote;
+        while (index != length) {
+            customNote.append(characters[index].unicode());
+            ++index;
+        }
+        m_footnoteWriter->addTextNode(customNote);
     }
     m_footnoteWriter->endElement();//text:note-citation
     //start the body of the footnote
@@ -283,7 +310,6 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
     m_insideFootnote = false;
 
     QString contents = QString::fromUtf8(m_footnoteBuffer->buffer(), m_footnoteBuffer->buffer().size());
-    kDebug(30513) << "add footnote to Paragraph: " << contents;
     m_paragraph->addRunOfText(contents, 0, QString(""), m_parser->styleSheet());
 
     //cleanup
@@ -310,6 +336,50 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
     //varElem.appendChild( footnoteElem );
 }
 
+
+void KWordTextHandler::annotationFound( wvWare::UString characters, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                                        const wvWare::AnnotationFunctor& parseAnnotation)
+{
+    Q_UNUSED(chp);
+    m_insideAnnotation = true;
+
+    m_annotationBuffer = new QBuffer();
+    m_annotationBuffer->open(QIODevice::WriteOnly);
+    m_annotationWriter = new KoXmlWriter(m_annotationBuffer);
+
+    m_annotationWriter->startElement("office:annotation");
+
+    m_annotationWriter->startElement("dc:creator");
+    // XXX: get the creator from the .doc
+    m_annotationWriter->endElement();
+
+    m_annotationWriter->startElement("dc:date");
+    // XXX: get the date from the .doc
+    m_annotationWriter->endElement();
+
+    //save the state of tables & paragraphs because we'll get new ones in the annotation
+    saveState();
+    //signal Document to parse the annotation
+    emit annotationFound(new wvWare::AnnotationFunctor(parseAnnotation), 0);
+    //and now restore state
+    restoreState();
+
+    //end the elements
+    m_annotationWriter->endElement();//office:annotation
+
+    m_insideAnnotation = false;
+
+    QString contents = QString::fromUtf8(m_annotationBuffer->buffer(), m_annotationBuffer->buffer().size());
+    m_paragraph->addRunOfText(contents, 0, QString(""), m_parser->styleSheet());
+
+    //cleanup
+    delete m_annotationWriter;
+    m_annotationWriter = 0;
+    delete m_annotationBuffer;
+    m_annotationBuffer = 0;
+}
+
+
 //create an element for the variable
 QDomElement KWordTextHandler::insertVariable(int type, wvWare::SharedPtr<const wvWare::Word97::CHP> chp, const QString& format)
 {
@@ -406,6 +476,33 @@ void KWordTextHandler::pictureFound(const wvWare::PictureFunctor& pictureFunctor
 }
 #endif // IMAGE_IMPORT
 
+void KWordTextHandler::drawingFound(unsigned int globalCP)
+{
+    kDebug(30513);
+    //create temporary writer for the picture tags
+    m_insideDrawing = true;
+    QBuffer drawingBuffer;
+    drawingBuffer.open(QIODevice::WriteOnly);
+
+    m_drawingWriter = new KoXmlWriter(&drawingBuffer);
+
+    saveState();
+    emit
+    drawingFound(globalCP, m_drawingWriter);
+    restoreState();
+
+    //now add content to our current paragraph
+    QString contents = QString::fromUtf8(drawingBuffer.buffer(),
+            drawingBuffer.buffer().size());
+    m_paragraph->addRunOfText(contents, 0, QString(""), m_parser->styleSheet());
+
+    m_insideDrawing = false;
+
+    //cleanup
+    delete m_drawingWriter;
+    m_drawingWriter = 0;
+}
+
 QDomElement KWordTextHandler::insertAnchor(const QString& fsname)
 {
     Q_UNUSED(fsname);
@@ -451,9 +548,14 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
     bool inStylesDotXml = false;
     if (m_insideFootnote) {
         writer = m_footnoteWriter;
-    } else if (m_writingHeader) {
-        writer = m_headerWriter;
+    } else if (m_insideDrawing) {
+        writer = m_drawingWriter;
+    } else if (document()->writingHeader()) {
+        writer = document()->headerWriter();
         inStylesDotXml = true;
+    }
+    else if (m_insideAnnotation) {
+        writer = m_annotationWriter;
     } else {
         writer = m_bodyWriter;
     }
@@ -497,7 +599,7 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
         } else if (listInfo->lsid() == 1 && listInfo->numberFormat() == 255) {
             // Looks like a heading, so that'll be processed in Paragraph.
             kDebug(30513) << "found heading, pap().ilvl="
-            << paragraphProperties->pap().ilvl;
+                    << paragraphProperties->pap().ilvl;
             isHeading = true;
             outlineLevel = paragraphProperties->pap().ilvl + 1;
         } else {
@@ -532,16 +634,17 @@ void KWordTextHandler::paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphP
         m_paragraph->setParagraphStyle(paragraphStyle);
 
         //write the paragraph formatting
-        //KoGenStyle* paragraphStyle = new KoGenStyle(KoGenStyle::StyleAuto, "paragraph");
+        //KoGenStyle* paragraphStyle = new KoGenStyle(KoGenStyle::ParagraphAutoStyle, "paragraph");
         //writeLayout(*paragraphProperties, paragraphStyle, m_currentStyle, true, namedStyleName);
     } else {
         kWarning() << "paragraphProperties was NOT set";
     }
 
     //check to see if we need a master page name attribute
-    if (m_writeMasterStyleName && !m_writingHeader) {
-        m_paragraph->getOdfParagraphStyle()->addAttribute("style:master-page-name", m_masterStyleName);
-        m_writeMasterStyleName = false;
+    if (document()->writeMasterStyleName() && !document()->writingHeader()) {
+        m_paragraph->getOdfParagraphStyle()->addAttribute("style:master-page-name",
+							  document()->masterStyleName());
+        document()->set_writeMasterStyleName(false);
     }
 } //end paragraphStart()
 
@@ -558,12 +661,19 @@ void KWordTextHandler::paragraphEnd()
     if (m_insideFootnote) {
         kDebug(30513) << "writing a footnote";
         m_paragraph->writeToFile(m_footnoteWriter);
-    } else if (!m_writingHeader) {
+    } else if (m_insideAnnotation) {
+        kDebug(30513) << "writing an annotation";
+        m_paragraph->writeToFile(m_annotationWriter);
+    } else if (m_insideDrawing) {
+        kDebug(30513) << "writing an drawing";
+        m_paragraph->writeToFile(m_drawingWriter);
+    } else if (!document()->writingHeader()) {
         kDebug(30513) << "writing to body";
         m_paragraph->writeToFile(m_bodyWriter);
+
     } else {
         kDebug(30513) << "writing a header";
-        m_paragraph->writeToFile(m_headerWriter);
+        m_paragraph->writeToFile(document()->headerWriter());
     }
 
     delete m_paragraph;
@@ -583,8 +693,17 @@ void KWordTextHandler::fieldStart(const wvWare::FLD* fld, wvWare::SharedPtr<cons
     switch (m_fieldType) {
     case 26:
     case 33:
+    case 37: // TAB
+        if (m_hyperLinkActive) {
+            m_fieldType = 88;
+        } else {
+            kDebug(30513) << "can't process field, just outputting text into document...";
+            m_fieldType = -1; //set m_fieldType for unprocessed field
+        }
+        break;
     case 88:  // HyperLink
         kDebug(30513) << "processing field...";
+        m_hyperLinkActive = true;
         break;
     default:
         kDebug(30513) << "can't process field, just outputting text into document...";
@@ -621,21 +740,66 @@ void KWordTextHandler::fieldEnd(const wvWare::FLD* /*fld*/, wvWare::SharedPtr<co
         writer.endElement();
         break;
     case 88:  // HyperLink
-        if (m_hyperLinkList.size() == 2) {
-            writer.startElement("text:a");
-            writer.addAttribute("xlink:type", "simple");
+        QList<QString> fullList(m_hyperLinkList);
+        QList<QString> emptyList;
+        QString hyperlink;
 
-            // Remove unneeded word and '"' characters from the URL
-            QString urlStr = m_hyperLinkList[0].remove(" HYPERLINK ");
-            if (urlStr.startsWith("\""))
-                urlStr = urlStr.remove(0, 1);
-            if (urlStr.endsWith("\""))
-                urlStr = urlStr.remove(urlStr.length() - 1, 1);
+        for (int i = 0; i < fullList.size(); i++) {
+            if (fullList[i].contains("PAGEREF")) {
+                fullList.removeAt(i);
+            }
+        }
+
+        while (fullList.size() >= 1) {
+            emptyList.clear();
+            bool passThrough = true;
+            if (fullList.size() >= 4) {
+                if (fullList[0] == " " && fullList[1].contains("HYPERLINK") && fullList[2] == " ") {
+                    hyperlink = fullList[0] + fullList[1] + fullList[2];
+                    emptyList.append(hyperlink);
+                    emptyList.append(fullList[3]);
+                    for(int i = 0; i<4; i++) {
+                        fullList.removeAt(0);
+                    }
+                    passThrough = false;
+                }
+            }
+            if (fullList.size() >= 2 && fullList[0].contains("HYPERLINK") && passThrough) {
+                emptyList.append(fullList[0]);
+                emptyList.append(fullList[1]);
+                for(int i = 0; i<2; i++) {
+                    fullList.removeAt(0);
+                }
+            } else if (passThrough) {
+                emptyList.append(hyperlink);
+                emptyList.append(fullList[0]);
+                fullList.removeAt(0);
+            }
 
-            writer.addAttribute("xlink:href", QUrl(urlStr).toEncoded());
-            writer.addTextNode(m_hyperLinkList[1]);
-            writer.endElement();
             m_hyperLinkList.clear();
+            m_hyperLinkList.append(emptyList[0]);
+            m_hyperLinkList.append(emptyList[1]);
+            if (m_hyperLinkList.size() == 2) {
+                writer.startElement("text:a");
+                writer.addAttribute("xlink:type", "simple");
+
+                // Remove unneeded word and '"' or ' ' characters from the URL
+                QString urlStr = m_hyperLinkList[0].remove(" HYPERLINK ");
+                while(urlStr.startsWith("\"") || urlStr.startsWith(" "))
+                    urlStr = urlStr.remove(0, 1);
+                while(urlStr.endsWith("\"") || urlStr.endsWith(" "))
+                    urlStr = urlStr.remove(urlStr.length() - 1, 1);
+
+                writer.addAttribute("xlink:href", QUrl(urlStr).toEncoded());
+                if (m_hyperLinkList[1].contains("\t")) {
+                    writer.startElement("text:tab");
+                    writer.endElement();
+                }
+                writer.addTextNode(m_hyperLinkList[1]);
+                writer.endElement();
+                m_hyperLinkList.clear();
+                m_hyperLinkActive = false;
+            }
         }
         break;
     }
@@ -678,6 +842,7 @@ void KWordTextHandler::runOfText(const wvWare::UString& text, wvWare::SharedPtr<
     if (m_insideField && m_fieldAfterSeparator && (m_fieldType > 0)) {
         kDebug(30513) << "adding this text to field value.";
         m_fieldValue.append(newText);
+        m_fieldType = 0;
         return;
     }
 
@@ -686,11 +851,13 @@ void KWordTextHandler::runOfText(const wvWare::UString& text, wvWare::SharedPtr<
     // ftcAscii = (rgftc[0]) font for ASCII text
     QString fontName = getFont(chp->ftcAscii);
     if (!fontName.isEmpty()) {
-        m_mainStyles->addFontFace(KoFontFace(fontName));
+        m_mainStyles->insertFontFace(KoFontFace(fontName));
     }
 
-    //add text string and formatting style to m_paragraph
-    m_paragraph->addRunOfText(newText, chp, fontName, m_parser->styleSheet());
+    if(chp->fVanish != 1) { // only show text that is not hidden
+        //add text string and formatting style to m_paragraph
+        m_paragraph->addRunOfText(newText, chp, fontName, m_parser->styleSheet());
+    }
 
 } //end runOfText()
 
@@ -767,6 +934,9 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
     if (listInfo->lsid() == 1 && nfc == 255) {
         return false;
     }
+
+    m_usedListWriters.push(writer);		// put the currently used writer in the stack
+
     //process the different places we could be in a list
     if (m_currentListID == 0) {
         //we're starting a new list...
@@ -784,15 +954,15 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
             m_listStyleName = m_previousListStyleName;
         } else {
             //need to create a style for this list
-            KoGenStyle listStyle(KoGenStyle::StyleListAuto);
+            KoGenStyle listStyle(KoGenStyle::ListAutoStyle);
 
             // If we're writing to styles.xml, the list style needs to go
             // there as well.
-            if (m_writingHeader)
+            if (document()->writingHeader())
                 listStyle.setAutoStyleInStylesDotXml(true);
 
             // Write styleName to the text:list tag.
-            m_listStyleName = m_mainStyles->lookup(listStyle);
+            m_listStyleName = m_mainStyles->insert(listStyle);
             writer->addAttribute("text:style-name", m_listStyleName);
         }
 
@@ -808,17 +978,17 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
     } else if (pap.ilvl > m_currentListDepth) {
         //we're going to a new level in the list
         kDebug(30513) << "going to a new level in list" << m_currentListID;
+        //open a new <text:list> or more levels if needed
         m_currentListDepth++;
-        //open a new <text:list>
-        //
-        // TODO: How come we need only open one new level (see above)
         writer->startElement("text:list");
+        for (;m_currentListDepth < pap.ilvl; m_currentListDepth++) {
+            writer->startElement("text:list-item");
+            writer->startElement("text:list");
+        }
         //it's a new level, so we need to configure this level
         newListLevelStyle = true;
     } else if (pap.ilvl < m_currentListDepth) {
         // We're backing out a level in the list.
-        //
-        // TODO: How come we need only close one level (see above)
         kDebug(30513) << "backing out a level in list" << m_currentListID;
         m_currentListDepth--;
         //close the last <text:list-item of the level
@@ -827,6 +997,11 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
         writer->endElement();
         //close the <text:list-item> from the surrounding level
         writer->endElement();
+        for (;m_currentListDepth > pap.ilvl; m_currentListDepth--) {
+            //close <text:list> and <text:list-item> for any additional levels
+            writer->endElement();
+            writer->endElement();
+        }
     } else {
         kDebug(30513) << "just another item on the same level in the list";
         //close <text:list-item> from the previous item
@@ -853,13 +1028,13 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
                 unsigned int code = text[0].unicode();
                 if ((code & 0xFF00) == 0xF000)   // see wv2
                     code &= 0x00FF;
-                listStyleWriter.addAttribute("text:bullet-char", QString::QString(code).toUtf8());
+                listStyleWriter.addAttribute("text:bullet-char", QString(code).toUtf8());
             } else
                 kWarning(30513) << "Bullet with more than one character, not supported";
 
             listStyleWriter.startElement("style:list-level-properties");
-            //TODO this is just hardcoded for now
-            listStyleWriter.addAttribute("text:min-label-width", "0.25in");
+            if (listInfo->space())
+                listStyleWriter.addAttributePt("text:min-label-distance", listInfo->space()/20.0);
             listStyleWriter.endElement(); //style:list-level-properties
             //close element
             listStyleWriter.endElement(); //text:list-level-style-bullet
@@ -880,9 +1055,10 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
             // Now we need to parse the text, to try and convert msword's powerful list template stuff
             QString prefix, suffix;
             bool depthFound = false;
+            bool anyLevelFound = false;
             int displayLevels = 1;
             // We parse <0>.<2>.<1>. as "level 2 with suffix='.'" (no prefix)
-            // But "Section <0>)" has both prefix and suffix.
+            // But "Section <0>.<1>)" has both prefix and suffix.
             // The common case is <0>.<1>.<2> (display-levels=3)
             //loop through all of text
             //this just sets depthFound & displayLevels & the suffix & prefix
@@ -895,13 +1071,13 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
                             kWarning(30513) << "ilvl " << pap.ilvl << " found twice in listInfo text...";
                         else
                             depthFound = true;
-                        suffix.clear();
+                        suffix.clear(); // really should never do anthing so why is it here??
                     } else {
                         Q_ASSERT(ch < pap.ilvl);   // Can't see how level 1 would have a <0> in it...
                         if (ch < pap.ilvl)
                             ++displayLevels; // we found a 'parent level', to be displayed
-                        prefix.clear(); // get rid of previous prefixes
                     }
+                    anyLevelFound = true;
                 }
                 //if it's not a number < 10
                 else {
@@ -909,7 +1085,7 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
                     if (depthFound)
                         suffix += QChar(ch);
                     //or add it to prefix if we haven't
-                    else
+                    else if (!anyLevelFound)
                         prefix += QChar(ch);
                 }
             }
@@ -932,13 +1108,12 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
                 if (nfc == 5 && suffix.isEmpty())
                     suffix = '.';
                 kDebug(30513) << " prefix=" << prefix << " suffix=" << suffix;
-                //counterElement.setAttribute( "type", Conversion::numberFormatCode( nfc ) );
                 listStyleWriter.addAttribute("style:num-format", Conversion::numberFormatCode(nfc));
-                //counterElement.setAttribute( "lefttext", prefix );
                 listStyleWriter.addAttribute("style:num-prefix", prefix);
-                //counterElement.setAttribute( "righttext", suffix );
                 listStyleWriter.addAttribute("style:num-suffix", suffix);
-                //counterElement.setAttribute( "display-levels", displayLevels );
+                if (displayLevels > 1) {
+                    listStyleWriter.addAttribute("text:display-levels", displayLevels);
+                }
                 kDebug(30513) << "storing suffix" << suffix << " for depth" << depth;
                 m_listSuffixes[ depth ] = suffix;
             } else {
@@ -952,14 +1127,29 @@ bool KWordTextHandler::writeListInfo(KoXmlWriter* writer, const wvWare::Word97::
                 //counterElement.setAttribute( "restart", "true" );
             }
 
-            //listInfo->alignment() is not supported in KWord
             //listInfo->isLegal() hmm
             //listInfo->notRestarted() [by higher level of lists] not supported
             //listInfo->followingchar() ignored, it's always a space in KWord currently
             //*************************************
             listStyleWriter.startElement("style:list-level-properties");
-            //TODO this is just hardcoded for now
-            listStyleWriter.addAttribute("text:min-label-width", "0.25in");
+            switch (listInfo->alignment()) {
+            case 1:
+                listStyleWriter.addAttribute("fo:text-align", "center");
+                break;
+            case 2:
+                listStyleWriter.addAttribute("fo:text-align", "end");
+                break;
+            case 3:
+                listStyleWriter.addAttribute("fo:text-align", "justify");
+                break;
+            default:
+                break;
+            }
+
+            if (listInfo->space())
+                listStyleWriter.addAttributePt("text:min-label-distance", listInfo->space()/20.0);
+            if (listInfo->indent())
+                listStyleWriter.addAttributePt("text:min-label-width", listInfo->indent()/20.0);
             listStyleWriter.endElement(); //style:list-level-properties
             //close element
             listStyleWriter.endElement(); //text:list-level-style-number
@@ -984,15 +1174,7 @@ void KWordTextHandler::closeList()
     kDebug(30513);
     // Set the correct XML writer.
     //
-    // TODO create m_writer, and just keep it pointing to the current writer
-    KoXmlWriter *writer;
-    if (m_insideFootnote) {
-        writer = m_footnoteWriter;
-    } else if (m_writingHeader) {
-        writer = m_headerWriter;
-    } else {
-        writer = m_bodyWriter;
-    }
+    KoXmlWriter *writer = m_usedListWriters.pop();		// get the last used writer from stack
 
     //TODO should probably test this more, to make sure it does work this way
     //for level 0, we need to close the last item and the list
