@@ -2,6 +2,9 @@
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
    Copyright 2008-2009 Inge Wallin    <inge@lysator.liu.se>
+   Copyright (C) 2010 Carlos Licea    <carlos@kdab.com>
+   Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+     Contact: Suresh Chande suresh.chande@nokia.com
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -43,6 +46,15 @@
 #include "ChartProxyModel.h"
 #include "Axis.h"
 #include "PlotArea.h"
+#include "Surface.h"
+
+// KOffice
+#include <KoXmlNS.h>
+#include <KoOdfGraphicStyles.h>
+#include <KoXmlReader.h>
+#include <KoShapeLoadingContext.h>
+#include <KoOdfLoadingContext.h>
+#include <KoOdfWorkaround.h>
 
 
 using namespace KChart;
@@ -53,18 +65,26 @@ public:
     Private( DataSet *parent );
     ~Private();
     
-    void       updateSize();
-    void       updateDiagram() const;
-    ChartType  effectiveChartType() const;
-    bool       isValidDataPoint( const QPoint &point ) const;
-    QVariant   data( const CellRegion &region, int index ) const;
+    void         updateSize();
+    bool         hasOwnChartType() const;
+    ChartType    effectiveChartType() const;
+    bool         isValidDataPoint( const QPoint &point ) const;
+    QVariant     data( const CellRegion &region, int index ) const;
+
+    QBrush defaultBrush() const;
+    QBrush defaultBrush( int section ) const;
+
+    QPen defaultPen() const;
+
+    // Determines what sections of a cell region lie in rect
+    void sectionsInRect( const CellRegion &region, const QRect &rect,
+                         int &start, int &end ) const;
+    void dataChanged( KDChartModel::DataRole role, const QRect &rect ) const;
     
     DataSet      *parent;
     
     ChartType     chartType;
     ChartSubtype  chartSubType;
-    ChartType     globalChartType;
-    ChartSubtype  globalChartSubType;
 
     Axis *attachedAxis;
     bool showMeanValue;
@@ -79,8 +99,18 @@ public:
     qreal errorMargin;
     qreal lowerErrorLimit;
     qreal upperErrorLimit;
+    // Determines whether pen has been set
+    bool penIsSet;
+    // Determines whether brush has been set
+    bool brushIsSet;
     QPen pen;
     QBrush brush;
+    KDChart::PieAttributes pieAttributes;
+
+    QMap<int, QPen> pens;
+    QMap<int, QBrush> brushes;
+    QMap<int, KDChart::PieAttributes> sectionsPieAttributes;
+
     int num;
 
     // The different CellRegions for a dataset
@@ -89,6 +119,8 @@ public:
     CellRegion yDataRegion;     // normal y values
     CellRegion xDataRegion;     // x values -- only for scatter & bubble charts
     CellRegion customDataRegion;// used for bubble width in bubble charts
+    // FIXME: Remove category region from DataSet - this is not the place
+    // it belongs to.
     CellRegion categoryDataRegion; // x labels -- same for all datasets
     
     ChartProxyModel *model;
@@ -124,6 +156,8 @@ DataSet::Private::Private( DataSet *parent )
     attachedAxis = 0;
     size = 0;
     blockSignals = false;
+    penIsSet = false;
+    brushIsSet = false;
 }
 
 DataSet::Private::~Private()
@@ -137,15 +171,19 @@ void DataSet::Private::updateSize()
     newSize = qMax( newSize, yDataRegion.cellCount() );
     newSize = qMax( newSize, customDataRegion.cellCount() );
     newSize = qMax( newSize, categoryDataRegion.cellCount() );
-    
-    size = newSize;
-    // FIXME: The first test below can *never* fail because of the
-    //        assignment just above. (iw)
-    if ( newSize != size && !blockSignals && kdChartModel ) {
-        kdChartModel->dataSetSizeChanged( parent, size );
+
+    if ( size != newSize ) {
+        size = newSize;
+        if ( !blockSignals && kdChartModel )
+            kdChartModel->dataSetSizeChanged( parent, size );
     }
 }
 
+bool DataSet::Private::hasOwnChartType() const
+{
+    return chartType != LastChartType;
+}
+
 /**
  * Returns the effective chart type of this data set, i.e.
  * returns the chart type of the diagram this data set is
@@ -154,25 +192,11 @@ void DataSet::Private::updateSize()
  */
 ChartType DataSet::Private::effectiveChartType() const
 {
-    if ( attachedAxis && chartType == LastChartType )
-        return attachedAxis->plotArea()->chartType();
-    return chartType;
-}
+    if ( hasOwnChartType() )
+        return chartType;
 
-void DataSet::Private::updateDiagram() const
-{
-    if ( kdDiagram && kdDataSetNumber >= 0 && size > 0 ) {
-
-	// FIXME: This should be done in a more proper way.
-	// The problem here is that line diagrams don't use the brush
-	// to paint their lines, but the pen. We on the other hand set
-	// the data set color on the brush, not the pen.
-        if ( effectiveChartType() == LineChartType )
-            kdDiagram->setPen( kdDataSetNumber, brush.color() );
-        else
-            kdDiagram->setPen( kdDataSetNumber, pen );
-        kdDiagram->setBrush( kdDataSetNumber, brush );
-    }
+    Q_ASSERT( attachedAxis );
+    return attachedAxis->plotArea()->chartType();
 }
 
 bool DataSet::Private::isValidDataPoint( const QPoint &point ) const
@@ -235,6 +259,99 @@ QVariant DataSet::Private::data( const CellRegion &region, int index ) const
     return data;
 }
 
+void DataSet::Private::sectionsInRect( const CellRegion &region, const QRect &rect,
+                                       int &start, int &end ) const
+{
+    QVector<QRect> dataRegions = region.rects();
+
+    start = -1;
+    end = -1;
+
+    if ( region.orientation() == Qt::Horizontal ) {
+        QPoint  topLeft  = rect.topLeft();
+        QPoint  topRight = rect.topRight();
+
+        int totalWidth = 0;
+        int i;
+
+        for ( i = 0; i < dataRegions.size(); i++ ) {
+            if ( dataRegions[i].contains( topLeft ) ) {
+                start = totalWidth + topLeft.x() - dataRegions[i].topLeft().x();
+                break;
+            }
+            totalWidth += dataRegions[i].width();
+        }
+
+        for ( ; i < dataRegions.size(); i++ ) {
+            if ( dataRegions[i].contains( topRight ) ) {
+                end = totalWidth + topRight.x() - dataRegions[i].topLeft().x();
+                break;
+            }
+
+            totalWidth += dataRegions[i].width();
+        }
+    }
+    else {
+        QPoint  topLeft    = rect.topLeft();
+        QPoint  bottomLeft = rect.bottomLeft();
+
+        int totalHeight = 0;
+        int i;
+        for ( i = 0; i < dataRegions.size(); i++ ) {
+            if ( dataRegions[i].contains( topLeft ) ) {
+                start = totalHeight + topLeft.y() - dataRegions[i].topLeft().y();
+                break;
+            }
+
+            totalHeight += dataRegions[i].height();
+        }
+
+        for ( ; i < dataRegions.size(); i++ ) {
+            if ( dataRegions[i].contains( bottomLeft ) ) {
+                end = totalHeight + bottomLeft.y() - dataRegions[i].topLeft().y();
+                break;
+            }
+
+            totalHeight += dataRegions[i].height();
+        }
+    }
+}
+
+QBrush DataSet::Private::defaultBrush() const
+{
+    Q_ASSERT( kdDiagram );
+    Qt::Orientation modelDataDirection = kdChartModel->dataDirection();
+    if ( modelDataDirection == Qt::Vertical )
+        return defaultDataSetColor( kdDataSetNumber );
+    // FIXME: What to return in the other case?
+    return QBrush();
+}
+
+QBrush DataSet::Private::defaultBrush( int section ) const
+{
+    Q_ASSERT( kdDiagram );
+    Qt::Orientation modelDataDirection = kdChartModel->dataDirection();
+    // Horizontally aligned diagrams have a specific color per category
+    // See for example pie or ring charts. A pie chart contains a single
+    // data set, but the slices default to different brushes.
+    if ( modelDataDirection == Qt::Horizontal )
+        return defaultDataSetColor( section );
+    // Vertically aligned diagrams default to one brush per data set
+    return defaultBrush();
+}
+
+QPen DataSet::Private::defaultPen() const
+{
+    QPen pen( Qt::black );
+    ChartType chartType = effectiveChartType();
+    if ( chartType == LineChartType ||
+         chartType == ScatterChartType )
+        pen = QPen( defaultDataSetColor( kdDataSetNumber ) );
+
+    return pen;
+}
+
+
 DataSet::DataSet( ChartProxyModel *proxyModel )
     : d( new Private( this ) )
 {
@@ -260,16 +377,6 @@ ChartSubtype DataSet::chartSubType() const
     return d->chartSubType;
 }
 
-ChartType DataSet::globalChartType() const
-{
-    return d->globalChartType;
-}
-
-ChartSubtype DataSet::globalChartSubType() const
-{
-    return d->globalChartSubType;
-}
-
 Axis *DataSet::attachedAxis() const
 {
     return d->attachedAxis;
@@ -338,11 +445,13 @@ void DataSet::setChartType( ChartType type )
         return;
     
     Axis  *axis = d->attachedAxis;
-    axis->detachDataSet( this );
+    if ( axis )
+        axis->detachDataSet( this );
     
     d->chartType = type;
     
-    axis->attachDataSet( this );
+    if ( axis )
+        axis->attachDataSet( this );
 }
 
 void DataSet::setChartSubType( ChartSubtype subType )
@@ -358,16 +467,6 @@ void DataSet::setChartSubType( ChartSubtype subType )
     axis->attachDataSet( this );
 }
 
-void DataSet::setGlobalChartType( ChartType type )
-{
-    d->globalChartType = type;
-}
-
-void DataSet::setGlobalChartSubType( ChartSubtype type )
-{
-    d->globalChartSubType = type;
-}
-
 
 void DataSet::setAttachedAxis( Axis *axis )
 {
@@ -412,28 +511,88 @@ void DataSet::setShowLabels( bool showLabels )
 
 QPen DataSet::pen() const
 {
-    return d->pen;
+    return d->penIsSet ? d->pen : d->defaultPen();
 }
 
 QBrush DataSet::brush() const
 {
-    return d->brush;
+    return d->brushIsSet ? d->brush : d->defaultBrush();
+}
+
+QPen DataSet::pen( int section ) const
+{
+    if ( d->pens.contains( section ) )
+        return d->pens[ section ];
+    return pen();
+}
+
+KDChart::PieAttributes DataSet::pieAttributes() const
+{
+    return d->pieAttributes;
+}
+
+QBrush DataSet::brush( int section ) const
+{
+    if ( d->brushes.contains( section ) )
+        return d->brushes[ section ];
+    if ( d->brushIsSet )
+        return brush();
+    return d->defaultBrush( section );
+}
+
+KDChart::PieAttributes DataSet::pieAttributes( int section ) const
+{
+    if( d->sectionsPieAttributes.contains( section ) )
+        return d->sectionsPieAttributes[ section ];
+    return pieAttributes();
 }
 
 void DataSet::setPen( const QPen &pen )
 {
     d->pen = pen;
-    d->updateDiagram();
-    if ( !d->blockSignals && d->attachedAxis )
-        d->attachedAxis->update();
+    d->penIsSet = true;
+    if ( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this );
 }
 
 void DataSet::setBrush( const QBrush &brush )
 {
     d->brush = brush;
-    d->updateDiagram();
-    if ( !d->blockSignals && d->attachedAxis )
-        d->attachedAxis->update();
+    d->brushIsSet = true;
+    if ( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this );
+}
+
+void DataSet::setPieExplodeFactor( int factor )
+{
+    KDChart::PieAttributes pieAttributes;
+    pieAttributes.setExplodeFactor( (qreal)factor / (qreal)100 );
+    d->pieAttributes = pieAttributes;
+    if( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this );
+}
+
+void DataSet::setPen( int section, const QPen &pen )
+{
+    d->pens[ section ] = pen;
+    if ( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::PenDataRole, section );
+}
+
+void DataSet::setBrush( int section, const QBrush &brush )
+{
+    d->brushes[ section ] = brush;
+    if ( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::BrushDataRole, section );
+}
+
+void DataSet::setPieExplodeFactor( int section, int factor )
+{
+    KDChart::PieAttributes pieAttributes;
+    pieAttributes.setExplodeFactor( (qreal)factor / (qreal)100 );
+    d->sectionsPieAttributes[ section ] = pieAttributes;
+    if( d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::PieAttributesRole, section);
 }
 
 QColor DataSet::color() const
@@ -537,7 +696,17 @@ QVariant DataSet::customData( int index ) const
 
 QVariant DataSet::categoryData( int index ) const
 {
-    return d->data( d->categoryDataRegion, index );
+    // There's no cell that holds this category's data
+    // (i.e., the region is either too short or simply empty)
+    if ( !d->categoryDataRegion.hasPointAtIndex( index ) )
+        return QString::number( index + 1 );
+ 
+    const QVariant data = d->data( d->categoryDataRegion, index );
+    // The cell contains valid data 
+    if ( data.isValid() )
+        return data;
+    // The cell is empty
+    return QString( "" );
 }
 
 QVariant DataSet::labelData() const
@@ -610,6 +779,9 @@ void DataSet::setXDataRegion( const CellRegion &region )
 {
     d->xDataRegion = region;
     d->updateSize();
+
+    if ( !d->blockSignals && d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::XDataRole, 0, size() - 1 );
 }
 
 void DataSet::setYDataRegion( const CellRegion &region )
@@ -617,16 +789,17 @@ void DataSet::setYDataRegion( const CellRegion &region )
     d->yDataRegion = region;
     d->updateSize();
     
-    if ( !d->blockSignals && d->kdChartModel ) {
-        d->kdChartModel->dataChanged( d->kdChartModel->index( 0,           d->kdDataSetNumber ),
-				      d->kdChartModel->index( d->size - 1, d->kdDataSetNumber ) );
-    }
+    if ( !d->blockSignals && d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::YDataRole, 0, size() - 1 );
 }
 
 void DataSet::setCustomDataRegion( const CellRegion &region )
 {
     d->customDataRegion = region;
     d->updateSize();
+
+    if ( !d->blockSignals && d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::CustomDataRole, 0, size() - 1 );
 }
 
 void DataSet::setCategoryDataRegion( const CellRegion &region )
@@ -639,6 +812,9 @@ void DataSet::setLabelDataRegion( const CellRegion &region )
 {
     d->labelDataRegion = region;
     d->updateSize();
+
+    if ( !d->blockSignals && d->kdChartModel )
+        d->kdChartModel->dataSetChanged( this, KDChartModel::LabelDataRole, 0, size() - 1 );
 }
 
 void DataSet::setXDataRegionString( const QString &string )
@@ -672,93 +848,69 @@ int DataSet::size() const
     return d->size > 0 ? d->size : 1;
 }
 
-void DataSet::yDataChanged( const QRect &rect ) const
-{    
-    int  start = -1;
-    int  end = -1;
-    
-    QVector<QRect> yDataRegionRects = d->yDataRegion.rects();
-    
-    if ( d->yDataRegion.orientation() == Qt::Horizontal ) {
-        QPoint  topLeft  = rect.topLeft();
-        QPoint  topRight = rect.topRight();
-        
-        int totalWidth = 0;
-        int i;
-        
-        for ( i = 0; i < yDataRegionRects.size(); i++ ) {
-            if ( yDataRegionRects[i].contains( topLeft ) ) {
-                start = totalWidth + topLeft.x() - yDataRegionRects[i].topLeft().x();
-                break;
-            }
-            totalWidth += yDataRegionRects[i].width();
-        }
-        
-        for ( ; i < yDataRegionRects.size(); i++ ) {
-            if ( yDataRegionRects[i].contains( topRight ) ) {
-                end = totalWidth + topRight.x() - yDataRegionRects[i].topLeft().x();
-                break;
-            }
+void DataSet::Private::dataChanged( KDChartModel::DataRole role, const QRect &rect ) const
+{
+    if ( blockSignals || !kdChartModel )
+        return;
 
-            totalWidth += yDataRegionRects[i].width();
-        }
+    const CellRegion *cellRegion = 0;
+    switch ( role ) {
+    case KDChartModel::YDataRole:
+        cellRegion = &yDataRegion;
+        break;
+    case KDChartModel::XDataRole:
+        cellRegion = &xDataRegion;
+        break;
+    case KDChartModel::CategoryDataRole:
+        cellRegion = &categoryDataRegion;
+        break;
+    case KDChartModel::LabelDataRole:
+        cellRegion = &labelDataRegion;
+        break;
+    case KDChartModel::CustomDataRole:
+        cellRegion = &customDataRegion;
+        break;
+    // TODO
+    case KDChartModel::ZDataRole:
+    case KDChartModel::BrushDataRole:
+    case KDChartModel::PenDataRole:
+        return;
     }
-    else {
-        QPoint  topLeft    = rect.topLeft();
-        QPoint  bottomLeft = rect.bottomLeft();
-        
-        int totalHeight = 0;
-        int i;
-        for ( i = 0; i < yDataRegionRects.size(); i++ ) {
-            if ( yDataRegionRects[i].contains( topLeft ) ) {
-                start = totalHeight + topLeft.y() - yDataRegionRects[i].topLeft().y();
-                break;
-            }
 
-            totalHeight += yDataRegionRects[i].height();
-        }
-        
-        for ( ; i < yDataRegionRects.size(); i++ ) {
-            if ( yDataRegionRects[i].contains( bottomLeft ) ) {
-                end = totalHeight + bottomLeft.y() - yDataRegionRects[i].topLeft().y();
-                break;
-            }
+    int start, end;
+    sectionsInRect( *cellRegion, rect, start, end );
 
-            totalHeight += yDataRegionRects[i].height();
-        }
-    }
-    
-    if ( !d->blockSignals && d->kdChartModel ) {
-        d->kdChartModel->dataChanged( d->kdChartModel->index( start, d->kdDataSetNumber ),
-                                     d->kdChartModel->index( end,   d->kdDataSetNumber ) );
-    }
+    kdChartModel->dataSetChanged( parent, role, start, end );
+}
+
+void DataSet::yDataChanged( const QRect &region ) const
+{
+    d->dataChanged( KDChartModel::YDataRole, region );
 }
 
 void DataSet::xDataChanged( const QRect &region ) const
 {
-    Q_UNUSED( region );
+    d->dataChanged( KDChartModel::XDataRole, region );
 }
 
 void DataSet::customDataChanged( const QRect &region ) const
 {
-    Q_UNUSED( region );
+    d->dataChanged( KDChartModel::CustomDataRole, region );
 }
 
 void DataSet::labelDataChanged( const QRect &region ) const
 {
-    Q_UNUSED( region );
+    d->dataChanged( KDChartModel::LabelDataRole, region );
 }
 
 void DataSet::categoryDataChanged( const QRect &region ) const
 {
-    Q_UNUSED( region );
+    d->dataChanged( KDChartModel::CategoryDataRole, region );
 }
 
 int DataSet::dimension() const
 {
-    const ChartType chartType = ( d->chartType != LastChartType
-				  ? d->chartType
-				  : d->globalChartType );
+    const ChartType chartType = d->effectiveChartType();
     // FIXME BUG: Ring, Surface
     switch ( chartType ) {
     case BarChartType:
@@ -807,8 +959,10 @@ int DataSet::kdDataSetNumber() const
 
 void DataSet::setKdDataSetNumber( int number )
 {
+    // FIXME: Is there anything to emit here?
+    // In theory, this should be done before any data is retrieved
+    // from KDChartModel
     d->kdDataSetNumber = number;
-    d->updateDiagram();
 }
 
 void DataSet::setKdChartModel( KDChartModel *model )
@@ -825,3 +979,124 @@ void DataSet::blockSignals( bool block )
 {
     d->blockSignals = block;
 }
+
+bool loadBrushAndPen(KoShapeLoadingContext &context, const KoXmlElement &n, QBrush& brush, bool& brushLoaded, QPen& pen, bool& penLoaded)
+{
+    if ( ! n.hasAttributeNS( KoXmlNS::chart, "style-name" ) )
+        return false;
+
+    KoOdfLoadingContext &odfLoadingContext = context.odfLoadingContext();
+    KoStyleStack &styleStack = odfLoadingContext.styleStack();
+    styleStack.save();
+    styleStack.clear();
+    odfLoadingContext.fillStyleStack( n, KoXmlNS::chart, "style-name", "chart" );
+
+    brushLoaded = false;                                                                                                                                                 
+    penLoaded = false; 
+
+    styleStack.setTypeProperties( "graphic" );
+
+    if ( styleStack.hasProperty( KoXmlNS::draw, "stroke" ) ) {
+        QString stroke = styleStack.property( KoXmlNS::draw, "stroke" );
+        pen = KoOdfGraphicStyles::loadOdfStrokeStyle( styleStack, stroke, odfLoadingContext.stylesReader() );
+        penLoaded = true;
+    }
+
+    if ( styleStack.hasProperty( KoXmlNS::draw, "fill" ) ) {
+        QString fill = styleStack.property( KoXmlNS::draw, "fill" );
+        if ( fill == "solid" || fill == "hatch" ) {
+            brush = KoOdfGraphicStyles::loadOdfFillStyle( styleStack, fill, odfLoadingContext.stylesReader() );
+            brushLoaded = true;
+        } else if ( fill == "gradient" ) {
+            brush = KoOdfGraphicStyles::loadOdfGradientStyle( styleStack, odfLoadingContext.stylesReader(), QSizeF( 5.0, 60.0 ) );
+            brushLoaded = true;
+        } else if ( fill == "bitmap" ) {
+            brush = Surface::loadOdfPatternStyle( styleStack, odfLoadingContext, QSizeF( 5.0, 60.0 ) );
+            brushLoaded = true;
+        }
+    }
+    
+#ifndef NWORKAROUND_ODF_BUGS
+    if( ! penLoaded) {
+        QColor fixedColor = KoOdfWorkaround::fixMissingStrokeColor( n, context );
+        if ( fixedColor.isValid() ) {
+            pen = fixedColor;
+            penLoaded = true;
+        }
+    }
+    if( ! brushLoaded) {
+        QColor fixedColor = KoOdfWorkaround::fixMissingFillColor( n, context );
+        if ( fixedColor.isValid() ) {
+            brush = fixedColor;
+            brushLoaded = true;
+        }
+    }
+#endif
+
+    styleStack.restore();
+    return true;
+}
+
+bool DataSet::loadOdf( const KoXmlElement &n,
+                       KoShapeLoadingContext &context )
+{
+    KoOdfLoadingContext &odfLoadingContext = context.odfLoadingContext();
+    KoStyleStack &styleStack = odfLoadingContext.styleStack();
+
+    {
+        QBrush brush(Qt::NoBrush);
+        QPen pen(Qt::NoPen);
+        bool brushLoaded = false;
+        bool penLoaded = false;
+        loadBrushAndPen(context, n, brush, brushLoaded, pen, penLoaded);
+        if(penLoaded)
+            setPen( pen );
+        if(brushLoaded)
+            setBrush( brush );
+        styleStack.save();
+        styleStack.setTypeProperties("chart");
+        if(styleStack.hasProperty(KoXmlNS::chart, "pie-offset"))
+            setPieExplodeFactor( styleStack.property( KoXmlNS::chart, "pie-offset" ).toInt() );
+        styleStack.restore();
+    }
+
+    if ( n.hasAttributeNS( KoXmlNS::chart, "values-cell-range-address" ) ) {
+        const QString region = n.attributeNS( KoXmlNS::chart, "values-cell-range-address", QString() );
+        setYDataRegionString( region );
+    }
+    if ( n.hasAttributeNS( KoXmlNS::chart, "label-cell-address" ) ) {
+        const QString region = n.attributeNS( KoXmlNS::chart, "label-cell-address", QString() );
+        setLabelDataRegionString( region );
+    }
+
+    // load data points
+    KoXmlElement m;
+    int loadedDataPointCount = 0;
+    forEachElement ( m, n ) {
+        if ( m.namespaceURI() != KoXmlNS::chart )
+            continue;
+        if ( m.localName() != "data-point" )
+            continue;
+        QBrush brush(Qt::NoBrush);
+        QPen pen(Qt::NoPen);
+        bool brushLoaded = false;
+        bool penLoaded = false;
+        loadBrushAndPen(context, m, brush, brushLoaded, pen, penLoaded);
+        if(penLoaded)
+            setPen( loadedDataPointCount, pen );
+        if(brushLoaded)
+            setBrush( loadedDataPointCount, brush );
+
+        //load pie explode factor
+        styleStack.save();
+        odfLoadingContext.fillStyleStack(m, KoXmlNS::chart, "style-name", "chart");
+        styleStack.setTypeProperties("chart");
+        if(styleStack.hasProperty( KoXmlNS::chart, "pie-offset"))
+            setPieExplodeFactor( loadedDataPointCount, styleStack.property( KoXmlNS::chart, "pie-offset" ).toInt() );
+        styleStack.restore();
+
+        ++loadedDataPointCount;
+    }
+
+    return true;
+}
