@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
-   Copyright 2009      Inge Wallin <inge@lysator.liu.se>
+   Copyright 2009-2010 Inge Wallin <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -80,12 +80,17 @@ using namespace KChart;
 
 const int MAX_PIXMAP_SIZE = 1000;
 
+Q_DECLARE_METATYPE( QPointer<QAbstractItemModel> )
+
 class PlotArea::Private
 {
 public:
-    Private( ChartShape *parent );
+    Private( PlotArea *q, ChartShape *parent );
     ~Private();
 
+    void initAxes();
+
+    PlotArea *q;
     // The parent chart shape
     ChartShape *shape;
 
@@ -109,13 +114,22 @@ public:
     // ----------------------------------------------------------------
     // Data specific to each types
 
+    // table:cell-range-address, ODF v1.2, $18.595
+    CellRegion cellRangeAddress;
+
     // 1. Bar charts
+    // FIXME: OpenOffice stores these attributes in the axes' elements.
+    // The specs don't say anything at all about what elements can have
+    // these style attributes.
+    // chart:vertical attribute: see ODF v1.2, $19.63
     bool  vertical;
     int   gapBetweenBars;
     int   gapBetweenSets;
     
     // 2. Pie charts
     int   pieExplodeFactor;     // in percents
+    // TODO: Load+Save
+    qreal pieAngleOffset;       // in degrees
 
     // ----------------------------------------------------------------
     // The embedded KD Chart
@@ -133,10 +147,10 @@ public:
     mutable bool pixmapRepaintRequested;
 };
 
-PlotArea::Private::Private( ChartShape *parent )
+PlotArea::Private::Private( PlotArea *q, ChartShape *parent )
+    : q(q)
+    , shape(parent)
 {
-    shape = parent;
-
     // Default type: normal bar chart
     chartType    = BarChartType;
     chartSubtype = NormalChartSubtype;
@@ -144,6 +158,9 @@ PlotArea::Private::Private( ChartShape *parent )
     wall  = 0;
     floor = 0;
 
+    // By default, x and y axes are not swapped.
+    vertical = false;
+
     threeD      = false;
     threeDScene = 0;
 
@@ -152,6 +169,10 @@ PlotArea::Private::Private( ChartShape *parent )
     gapBetweenBars = 0;
     gapBetweenSets = 100;
 
+    // OpenOffice.org's default. It means the first pie slice starts at the
+    // very top (and then going counter-clockwise).
+    pieAngleOffset = 90.0;
+
     // KD Chart stuff
     kdChart = new KDChart::Chart();
     kdPlane = new KDChart::CartesianCoordinatePlane( kdChart );
@@ -167,11 +188,34 @@ PlotArea::Private::~Private()
     delete kdChart;
 }
 
+void PlotArea::Private::initAxes()
+{
+    // The category data region is anchored to an axis and will be set on addAxis if the
+    // axis defines the Axis::categoryDataRegionString(). So, clear it now.
+    q->proxyModel()->setCategoryDataRegion(QString());
+    // Remove all old axes
+    while( !axes.isEmpty() ) {
+        Axis *axis = axes.takeLast();
+        Q_ASSERT( axis );
+        if ( axis->title() )
+            automaticallyHiddenAxisTitles.removeAll( axis->title() );
+        delete axis;
+    }
+    // There need to be at least these two axes. Do not delete, but
+    // hide them instead.
+    Axis *xAxis = new Axis( q );
+    Axis *yAxis = new Axis( q );
+    xAxis->setPosition( BottomAxisPosition );
+    yAxis->setPosition( LeftAxisPosition );
+    yAxis->setShowMajorGrid( true );
+    axes.append( xAxis );
+    axes.append( yAxis );
+}
 
 PlotArea::PlotArea( ChartShape *parent )
     : QObject()
     , KoShape()
-    , d( new Private( parent ) )
+    , d( new Private( this, parent ) )
 {
     setShapeId( ChartShapeId );
     
@@ -210,15 +254,7 @@ void PlotArea::plotAreaInit()
     d->wall = new Surface( this );
     //d->floor = new Surface( this );
     
-    // There need to be at least these two axes. Do not delete, but
-    // hide them instead.
-    Axis *xAxis = new Axis( this );
-    Axis *yAxis = new Axis( this );
-    xAxis->setPosition( BottomAxisPosition );
-    yAxis->setPosition( LeftAxisPosition );
-    yAxis->setShowMajorGrid( true );
-    d->axes.append( xAxis );
-    d->axes.append( yAxis );
+    d->initAxes();
 }
 
 void PlotArea::dataSetCountChanged()
@@ -321,6 +357,16 @@ bool PlotArea::isThreeD() const
     return d->threeD;
 }
 
+CellRegion PlotArea::cellRangeAddress() const
+{
+    return d->cellRangeAddress;
+}
+
+bool PlotArea::isVertical() const
+{
+    return d->vertical;
+}
+
 ThreeDScene *PlotArea::threeDScene() const
 {
     return d->threeDScene;
@@ -336,6 +382,11 @@ int PlotArea::gapBetweenSets() const
     return d->gapBetweenSets;
 }
 
+qreal PlotArea::pieAngleOffset() const
+{
+    return d->pieAngleOffset;
+}
+
 bool PlotArea::addAxis( Axis *axis )
 {
     if ( d->axes.contains( axis ) ) {
@@ -350,8 +401,16 @@ bool PlotArea::addAxis( Axis *axis )
     d->axes.append( axis );
     
     if ( axis->dimension() == XAxisDimension ) {
-        foreach ( Axis *_axis, d->axes )
-            _axis->registerKdXAxis( axis->kdAxis() );
+        // set the categoryDataRegion of the proxyModel. This will then be used on
+        // ChartProxyModel::createDataSetsFromRegion to create the dataSets.
+        if ( proxyModel()->categoryDataRegion().isEmpty() && ! axis->categoryDataRegionString().isEmpty() )
+            proxyModel()->setCategoryDataRegion(axis->categoryDataRegionString());
+
+        // let each axis know about the other axis
+        foreach ( Axis *_axis, d->axes ) {
+            if ( _axis->isVisible() )
+                _axis->registerKdAxis( axis->kdAxis() );
+        }
     }
     
     requestRepaint();
@@ -370,11 +429,26 @@ bool PlotArea::removeAxis( Axis *axis )
     	qWarning() << "PlotArea::removeAxis(): Pointer to axis is NULL!";
     	return false;
     }
+    
+    if ( axis->title() )
+        d->automaticallyHiddenAxisTitles.removeAll( axis->title() );
+
     d->axes.removeAll( axis );
     
     if ( axis->dimension() == XAxisDimension ) {
+        // If the axis is removed we probably need to update the used categoryDataRegion too.
+        if ( ! proxyModel()->categoryDataRegion().isEmpty() && proxyModel()->categoryDataRegion() == axis->categoryDataRegionString() ) {
+            proxyModel()->setCategoryDataRegion(QString());
+            foreach ( Axis *_axis, d->axes ) {
+                 if ( _axis->dimension() == XAxisDimension && ! _axis->categoryDataRegionString().isEmpty()) {
+                     proxyModel()->setCategoryDataRegion( _axis->categoryDataRegionString() );
+                     break;
+                 }
+            }
+        }
+        
         foreach ( Axis *_axis, d->axes )
-            _axis->deregisterKdXAxis( axis->kdAxis() );
+            _axis->deregisterKdAxis( axis->kdAxis() );
     }
     
     // This also removes the axis' title, which is a shape as well
@@ -448,6 +522,16 @@ void PlotArea::setThreeD( bool threeD )
     requestRepaint();
 }
 
+void PlotArea::setCellRangeAddress( const CellRegion &region )
+{
+    d->cellRangeAddress = region;
+}
+
+void PlotArea::setVertical( bool vertical )
+{
+    d->vertical = vertical;
+    // TODO: Propagate
+}
 
 // ----------------------------------------------------------------
 //                         loading and saving
@@ -461,16 +545,41 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
 
     styleStack.clear();
 
+    // First step is to load the axis. Datasets are attached to an
+    // axis and we need the axis to check for categories.
+    d->initAxes();
+    KoXmlElement n;
+    forEachElement ( n, plotAreaElement ) {
+        if ( n.namespaceURI() != KoXmlNS::chart )
+            continue;
+
+        if ( n.localName() == "axis" ) {
+            Axis *axis = new Axis( this );
+            axis->loadOdf( n, context );
+            addAxis( axis );
+        }
+    }
+
+    CellRegion cellRangeAddress;
+    if ( plotAreaElement.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) )
+    {
+        cellRangeAddress = CellRegion( plotAreaElement.attributeNS( KoXmlNS::table, "cell-range-address" ) );
+    }
+
     // Find out about things that are in the plotarea style.
     // 
     // These things include chart subtype, special things for some
     // chart types like line charts, stock charts, etc.
+    QString seriesSource;
     if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
         context.odfLoadingContext().fillStyleStack( plotAreaElement, KoXmlNS::chart, "style-name", "chart" );
 
         styleStack.setTypeProperties( "graphic" );
         styleStack.setTypeProperties( "chart" );
 
+        if ( styleStack.hasProperty( KoXmlNS::chart, "three-dimensional" ) )
+            setThreeD( styleStack.property( KoXmlNS::chart, "three-dimensional" ) == "true" );
+
         // Set subtypes stacked or percent.
         // These are valid for Bar, Line, Area and Radar types.
         if ( styleStack.hasProperty( KoXmlNS::chart, "percentage" )
@@ -485,25 +594,13 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         }
 
         // Data specific to bar charts
-        if ( styleStack.hasProperty( KoXmlNS::chart, "vertical" )
-             && styleStack.property( KoXmlNS::chart, "vertical" ) == "true" )
-        {
-            d->vertical = true;
-        }
+        if ( styleStack.hasProperty( KoXmlNS::chart, "vertical" ) )
+            setVertical( styleStack.property( KoXmlNS::chart, "vertical" ) == "true" );
 
         // Data direction: It's in the plotarea style.
         if ( styleStack.hasProperty( KoXmlNS::chart, "series-source" ) ) {
-            const QString  seriesSource
-                = styleStack.property( KoXmlNS::chart, "series-source" );
-
+            seriesSource = styleStack.property( KoXmlNS::chart, "series-source" );
             kDebug(35001) << "series-source=" << seriesSource;
-            if ( seriesSource == "rows" )
-                proxyModel()->setDataDirection( Qt::Horizontal );
-            else if ( seriesSource == "columns" )
-                proxyModel()->setDataDirection( Qt::Vertical );
-            else
-                // Use the default value for wrong values (not "rows" or "columns")
-                proxyModel()->setDataDirection( Qt::Vertical );
         }
 
         // Special properties for various chart types
@@ -516,19 +613,26 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
 #endif
     }
 
+    if ( seriesSource == "rows" )
+        proxyModel()->setDataDirection( Qt::Horizontal );
+    else if ( seriesSource == "columns" )
+        proxyModel()->setDataDirection( Qt::Vertical );
+    else
+        // By default, OOo creates consecutive data series column-wise;
+        // by adopting that behaviour we avoid extra special-handling of
+        // documents created by OOo.
+        proxyModel()->setDataDirection( Qt::Vertical );
+
     loadOdfAttributes( plotAreaElement, context, OdfAllAttributes );
     
-    //KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
-    
     // Find out if the data table contains labels as first row and/or column.
     // This is in the plot-area element itself.
-    if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart,
-                                         "data-source-has-labels" ) ) {
-
+    if( proxyModel()->categoryDataRegion().isEmpty() && // if chart:categories with a table:cell-range-address is defined within an axis then we need to ignore the data-source-has-labels.
+        plotAreaElement.hasAttributeNS( KoXmlNS::chart, "data-source-has-labels" ) ) {
         // Yes, it does.  Now find out how.
         const QString  dataSourceHasLabels
             = plotAreaElement.attributeNS( KoXmlNS::chart,
-                                           "data-source-has-labels" );
+                                        "data-source-has-labels" );
         if ( dataSourceHasLabels == "both" ) {
             proxyModel()->setFirstRowIsLabel( true );
             proxyModel()->setFirstColumnIsLabel( true );
@@ -550,32 +654,38 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
         proxyModel()->setFirstColumnIsLabel( false );
     }
 
-    // Remove all axes before loading new ones
-    while( !d->axes.isEmpty() ) {
-        Axis *axis = d->axes.takeLast();
-        Q_ASSERT( axis );
-        delete axis;
+    QAbstractItemModel *sheetAccessModel = 0;
+    if (d->shape->resourceManager()->hasResource(75751149)) { // duplicated from kspread
+        QVariant var = d->shape->resourceManager()->resource(75751149);
+        sheetAccessModel = static_cast<QAbstractItemModel*>(var.value<void*>());
+    }
+
+    setCellRangeAddress( cellRangeAddress );
+
+    if ( sheetAccessModel )
+    {
+        const QString sheetName = cellRangeAddress.sheetName();
+        int sheetIndex = 0;
+        // Find sheet that this cell range address is associated with
+        if ( !sheetName.isEmpty() ) {
+            while ( sheetIndex + 1 < sheetAccessModel->columnCount() &&
+                    sheetAccessModel->headerData( sheetIndex, Qt::Horizontal ) != sheetName )
+                sheetIndex++;
+        }
+        QPointer<QAbstractItemModel> sheet = sheetAccessModel->data( sheetAccessModel->index( 0, sheetIndex ) ).value< QPointer<QAbstractItemModel> >();
+
+        // If sheet can't be found, we'll stay with the back-up model loaded from the
+        // chart document.
+        if ( sheet )
+            d->shape->setModel( sheet.data() );
     }
     
-    // A data set is always attached to an axis, so load them first.
-    KoXmlElement n;
-    forEachElement ( n, plotAreaElement ) {
-        if ( n.namespaceURI() != KoXmlNS::chart )
-            continue;
-
-        if ( n.localName() == "axis" ) {
-            Axis *axis = new Axis( this );
-            axis->loadOdf( n, context );
-            addAxis( axis );
-        }
-    }
-
     // Now, after the axes, load the datasets.
     // Note that this only contains properties of the datasets, the
     // actual data is not stored here.
     //
     // FIXME: Isn't the proxy model a strange place to store this data?
-    d->shape->proxyModel()->loadOdf( plotAreaElement, context );
+    proxyModel()->loadOdf( plotAreaElement, context );
 
     // Now load the surfaces (wall and possibly floor)
     // FIXME: Use named tags instead of looping?
@@ -614,7 +724,7 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     
     // FIXME: Somehow this style gets the name gr2 instead of ch2.
     //        Fix that as well.
-    KoGenStyle plotAreaStyle( KoGenStyle::StyleChartAuto, "chart" );
+    KoGenStyle plotAreaStyle( KoGenStyle::ChartAutoStyle, "chart" );
     
     // Data direction
     const Qt::Orientation direction = proxyModel()->dataDirection();
@@ -634,6 +744,8 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     bodyWriter.addAttributePt( "svg:x", p.x() );
     bodyWriter.addAttributePt( "svg:y", p.y() );
 
+    bodyWriter.addAttribute( "table:cell-range-address", d->cellRangeAddress.toString() );
+
     // About the data:
     //   Save if the first row / column contain headers.
     QString  dataSourceHasLabels;
@@ -725,7 +837,7 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
         switch( d->chartSubtype ) {
         case NoChartSubtype:
         case NormalChartSubtype:
-        break;
+            break;
         case StackedChartSubtype:
             plotAreaStyle.addProperty( "chart:stacked", "true" );
             break;
@@ -738,8 +850,7 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
             plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
             // FIXME: Save all 3D attributes too.
         }
-
-    break;
+        break;
 
     case CircleChartType:
         // FIXME
@@ -804,6 +915,13 @@ void PlotArea::setPieExplodeFactor( DataSet *dataSet, int percent )
     emit pieExplodeFactorChanged( dataSet, percent );
 }
 
+void PlotArea::setPieAngleOffset( qreal angle )
+{
+    d->pieAngleOffset = angle;
+
+    emit pieAngleOffsetChanged( angle );
+}
+
 ChartShape *PlotArea::parent() const
 {
     // There has to be a valid parent
