@@ -159,7 +159,7 @@ QString SheetPrint::saveOdfSheetStyleLayout(KoGenStyles &mainStyles)
     // the style has to be in the styles.xml file and only there
     pageLayout.setAutoStyleInStylesDotXml(true);
 
-    return mainStyles.lookup(pageLayout, "pm");
+    return mainStyles.insert(pageLayout, "pm");
 }
 
 
@@ -213,6 +213,13 @@ bool SheetPrint::pageNeedsPrinting(QRect& page_range)
     return false;
 }
 
+void SheetPrint::generateThumbnail(QPainter &painter)
+{
+    painter.scale(m_zoomHandler->zoomedResolutionX(), m_zoomHandler->zoomedResolutionY());
+    painter.setClipRect(0, 0, (int)paperLayout().width, (int)paperLayout().height);
+    printPage(painter, cellsPrintRange(), QRect(0, 0, paperLayout().width, paperLayout().height), QPointF(0,0));
+}
+
 bool SheetPrint::print(QPainter &painter, QPrinter *_printer)
 {
     kDebug(36001) << "PRINTING ....";
