@@ -197,7 +197,7 @@ bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, cons
         switch (id) {
         case KoParagraphStyle::UnnumberedListItem:
         case KoParagraphStyle::IsListHeader:
-            if (actualProperty[id].toInt() != expectedProperty[id].toBool())
+            if (actualProperty[id].toInt() != static_cast<int>(expectedProperty[id].toBool()))
                 match = false;
             break;
         case KoParagraphStyle::AutoTextIndent:
@@ -913,10 +913,9 @@ QTextDocument *TestLoading::documentFromOdt(const QString &odt)
     KoChangeTracker *changeTracker = new KoChangeTracker;
 
     KoOdfLoadingContext odfLoadingContext(odfReadStore.styles(), odfReadStore.store(), *componentData);
-    QMap<QString, KoDataCenter *> dataCenterMap;
-    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, dataCenterMap);
+    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, 0);
     KoTextSharedLoadingData *textSharedLoadingData = new KoTextSharedLoadingData;
-    textSharedLoadingData->loadOdfStyles(odfLoadingContext, styleManager);
+    textSharedLoadingData->loadOdfStyles(shapeLoadingContext, styleManager);
     shapeLoadingContext.addSharedData(KOTEXT_SHARED_LOADING_ID, textSharedLoadingData);
 
     KoTextShapeData *textShapeData = new KoTextShapeData;
@@ -1006,7 +1005,7 @@ QString TestLoading::documentToOdt(QTextDocument *document)
 
     contentTmpFile.close();
 
-    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, contentWriter);
 
     contentWriter->startElement("office:body");
     contentWriter->startElement("office:text");
