@@ -24,6 +24,7 @@
 
 #include <QTextBlock>
 #include <QTextCursor>
+#include <QFontMetrics>
 
 #include <KoOdfLoadingContext.h>
 #include <KoOdfStylesReader.h>
@@ -164,8 +165,7 @@ QPen KoCharacterStyle::textOutline() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::TextOutline);
     if (variant.isNull()) {
-        QPen pen(Qt::NoPen);
-        return pen;
+        return QPen(Qt::NoPen);
     }
     return qvariant_cast<QPen>(variant);
 }
@@ -175,8 +175,7 @@ QBrush KoCharacterStyle::background() const
     QVariant variant = d->stylesPrivate.value(QTextFormat::BackgroundBrush);
 
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -190,8 +189,7 @@ QBrush KoCharacterStyle::foreground() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::ForegroundBrush);
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -289,8 +287,10 @@ static void importOdfLine(const QString &type, const QString &style, const QStri
         fixedType.clear();
     else if (fixedType.isEmpty() && !fixedStyle.isEmpty())
         fixedType = "single";
-    else if (!fixedType.isEmpty() && fixedStyle.isEmpty())
+    else if (!fixedType.isEmpty() && fixedType != "none" && fixedStyle.isEmpty()) {
+        // don't set a style when the type is none
         fixedStyle = "solid";
+    }
 
     if (fixedType == "single")
         lineType = KoCharacterStyle::SingleLine;
@@ -700,7 +700,7 @@ QFont::Capitalization KoCharacterStyle::fontCapitalization() const
 
 void KoCharacterStyle::setCountry(const QString &country)
 {
-    if (country.isNull())
+    if (country.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Country);
     else
         d->setProperty(KoCharacterStyle::Country, country);
@@ -708,7 +708,7 @@ void KoCharacterStyle::setCountry(const QString &country)
 
 void KoCharacterStyle::setLanguage(const QString &language)
 {
-    if (language.isNull())
+    if (language.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Language);
     else
         d->setProperty(KoCharacterStyle::Language, language);
@@ -744,7 +744,7 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext &context)
         if (fontFace != 0)
             fontName = fontFace->attributeNS(KoXmlNS::svg, "font-family", "");
     }
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
     // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -808,7 +808,7 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
     if (styleStack.hasProperty(KoXmlNS::style, "font-family"))
         fontName = styleStack.property(KoXmlNS::style, "font-family");
 
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
         // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -969,6 +969,16 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
             setVerticalAlignment(QTextCharFormat::AlignSuperScript);
         else if (textPosition.startsWith("sub"))
             setVerticalAlignment(QTextCharFormat::AlignSubScript);
+        else {
+            QRegExp re("(-?[\\d.]+)%.*");
+            if (re.exactMatch(textPosition)) {
+                float value = re.capturedTexts()[1].toFloat();
+                if (value > 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSuperScript);
+                else if (value < 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSubScript);
+            }
+        }
     }
 
     // The fo:font-variant attribute provides the option to display text as small capitalized letters.
@@ -1035,6 +1045,12 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
         }
     }
 
+    if (styleStack.hasProperty(KoXmlNS::fo, "letter-spacing")) {
+        qreal space = KoUnit::parseValue(styleStack.property(KoXmlNS::fo, "letter-spacing"));
+        QFontMetrics fm(font());
+        setFontLetterSpacing(100+100*space/fm.averageCharWidth());
+    }
+
     if (styleStack.hasProperty(KoXmlNS::style, "text-outline")) {
         if (styleStack.property(KoXmlNS::style, "text-outline") == "true") {
             setTextOutline(QPen((foreground().style() != Qt::NoBrush)?foreground():QBrush(Qt::black) , 0));
@@ -1227,6 +1243,8 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
             style.addProperty("fo:country", d->stylesPrivate.value(KoCharacterStyle::Country).toString(), KoGenStyle::TextType);
         } else if (key == KoCharacterStyle::Language) {
             style.addProperty("fo:language", d->stylesPrivate.value(KoCharacterStyle::Language).toString(), KoGenStyle::TextType);
+        } else if (key == QTextCharFormat::FontLetterSpacing) {
+            style.addProperty("fo:letter-spacing", (int) fontLetterSpacing(), KoGenStyle::TextType);
         } else if (key == QTextFormat::TextOutline) {
             QPen outline = textOutline();
             style.addProperty("style:text-outline", outline.style() == Qt::NoPen ? "false" : "true", KoGenStyle::TextType);
@@ -1242,4 +1260,9 @@ QVariant KoCharacterStyle::value(int key) const
     return d->stylesPrivate.value(key);
 }
 
+void KoCharacterStyle::removeHardCodedDefaults()
+{
+    d->hardCodedDefaultStyle.clearAll();
+}
+
 #include <KoCharacterStyle.moc>
