@@ -139,7 +139,7 @@ QVariant KoParagraphStyle::value(int key) const
 {
     QVariant var = d->stylesPrivate.value(key);
     if (var.isNull() && d->parentStyle)
-        var = d->parentStyle->value(key);
+        return d->parentStyle->value(key);
     return var;
 }
 
@@ -176,8 +176,7 @@ QColor KoParagraphStyle::propertyColor(int key) const
 {
     QVariant variant = value(key);
     if (variant.isNull()) {
-        QColor color;
-        return color;
+        return QColor();
     }
     return qvariant_cast<QColor>(variant);
 }
@@ -187,11 +186,15 @@ void KoParagraphStyle::applyStyle(QTextBlockFormat &format) const
     if (d->parentStyle) {
         d->parentStyle->applyStyle(format);
     }
+    format.clearProperty(QTextFormat::PageBreakPolicy); //this should not be inherited according to odf
     QList<int> keys = d->stylesPrivate.keys();
     for (int i = 0; i < keys.count(); i++) {
         QVariant variant = d->stylesPrivate.value(keys[i]);
         format.setProperty(keys[i], variant);
     }
+    if ((hasProperty(DefaultOutlineLevel)) && (!format.hasProperty(OutlineLevel))) {
+       format.setProperty(OutlineLevel, defaultOutlineLevel());
+    }
 }
 
 void KoParagraphStyle::applyStyle(QTextBlock &block, bool applyListStyle) const
@@ -379,22 +382,48 @@ bool KoParagraphStyle::followDocBaseline() const
 
 void KoParagraphStyle::setBreakBefore(bool on)
 {
-    setProperty(BreakBefore, on);
+    int prop = d->stylesPrivate.value(QTextFormat::PageBreakPolicy).toInt();
+
+    if(on) {
+        prop |= QTextFormat::PageBreak_AlwaysBefore;
+    } else {
+        prop &= ~QTextFormat::PageBreak_AlwaysBefore;
+    }
+
+    setProperty(QTextFormat::PageBreakPolicy, prop);
 }
 
 bool KoParagraphStyle::breakBefore()
 {
-    return propertyBoolean(BreakBefore);
+    //we shouldn't use propertyBoolean as this value is not inherited but default to false
+    QVariant var = d->stylesPrivate.value(QTextFormat::PageBreakPolicy);
+    if(var.isNull())
+        return false;
+
+    return var.toInt() & QTextFormat::PageBreak_AlwaysBefore;
 }
 
 void KoParagraphStyle::setBreakAfter(bool on)
 {
-    setProperty(BreakAfter, on);
+    int prop = d->stylesPrivate.value(QTextFormat::PageBreakPolicy).toInt();
+
+    if(on) {
+        prop |= QTextFormat::PageBreak_AlwaysAfter;
+    } else {
+        prop &= ~QTextFormat::PageBreak_AlwaysAfter;
+    }
+
+    setProperty(QTextFormat::PageBreakPolicy, prop);
 }
 
 bool KoParagraphStyle::breakAfter()
 {
-    return propertyBoolean(BreakAfter);
+    //we shouldn't use propertyBoolean as this value is not inherited but default to false
+    QVariant var = d->stylesPrivate.value(QTextFormat::PageBreakPolicy);
+    if(var.isNull())
+        return false;
+
+    return var.toInt() & QTextFormat::PageBreak_AlwaysAfter;
 }
 
 void KoParagraphStyle::setLeftPadding(qreal padding)
@@ -823,6 +852,16 @@ int KoParagraphStyle::outlineLevel() const
     return propertyInt(OutlineLevel);
 }
 
+void KoParagraphStyle::setDefaultOutlineLevel(int outline)
+{
+    setProperty(DefaultOutlineLevel, outline);
+}
+
+int KoParagraphStyle::defaultOutlineLevel() const
+{
+    return propertyInt(DefaultOutlineLevel);
+}
+
 void KoParagraphStyle::setIsListHeader(bool on)
 {
     setProperty(IsListHeader, on);
@@ -891,8 +930,7 @@ QBrush KoParagraphStyle::background() const
     QVariant variant = d->stylesPrivate.value(QTextFormat::BackgroundBrush);
 
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -905,12 +943,6 @@ void KoParagraphStyle::loadOdf(const KoXmlElement *element, KoOdfLoadingContext
     if (d->name.isEmpty()) // if no style:display-name is given us the style:name
         d->name = element->attributeNS(KoXmlNS::style, "name", QString());
 
-#if 0 //1.6:
-    // OOo hack:
-    //m_bOutline = name.startsWith( "Heading" );
-    // real OASIS solution:
-    bool m_bOutline = element->hasAttributeNS(KoXmlNS::style, "default-outline-level");
-#endif
     context.styleStack().save();
     // Load all parents - only because we don't support inheritance.
     QString family = element->attributeNS(KoXmlNS::style, "family", "paragraph");
@@ -919,10 +951,17 @@ void KoParagraphStyle::loadOdf(const KoXmlElement *element, KoOdfLoadingContext
     //setParent( d->stylemanager->defaultParagraphStyle() );
 
     QString masterPage = element->attributeNS(KoXmlNS::style, "master-page-name", QString());
-    if (! masterPage.isNull()) {
+    if (! masterPage.isEmpty()) {
         setMasterPageName(masterPage);
     }
 
+    if (element->hasAttributeNS(KoXmlNS::style, "default-outline-level")) {
+        bool ok = false;
+        int level = element->attributeNS(KoXmlNS::style, "default-outline-level").toInt(&ok);
+        if (ok)
+            setDefaultOutlineLevel(level);
+    }
+    
     //1.6: KoTextFormat::load
     KoCharacterStyle *charstyle = characterStyle();
     context.styleStack().setTypeProperties("text");   // load all style attributes from "style:text-properties"
@@ -1253,38 +1292,6 @@ void KoParagraphStyle::loadOdfProperties(KoStyleStack &styleStack)
         setDropCapsDistance(distance);
     }
 
-    // Page breaking
-#if 0
-    int pageBreaking = 0;
-    if (styleStack.hasProperty(KoXmlNS::fo, "break-before") ||
-            styleStack.hasProperty(KoXmlNS::fo, "break-after") ||
-            styleStack.hasProperty(KoXmlNS::fo, "keep-together") ||
-            styleStack.hasProperty(KoXmlNS::style, "keep-with-next") ||
-            styleStack.hasProperty(KoXmlNS::fo, "keep-with-next")) {
-        if (styleStack.hasProperty(KoXmlNS::fo, "break-before")) {    // 3.11.24
-            // TODO in KWord: implement difference between "column" and "page"
-            if (styleStack.property(KoXmlNS::fo, "break-before") != "auto")
-                pageBreaking |= KoParagLayout::HardFrameBreakBefore;
-        } else if (styleStack.hasProperty(KoXmlNS::fo, "break-after")) {   // 3.11.24
-            // TODO in KWord: implement difference between "column" and "page"
-            if (styleStack.property(KoXmlNS::fo, "break-after") != "auto")
-                pageBreaking |= KoParagLayout::HardFrameBreakAfter;
-        }
-
-        if (styleStack.hasProperty(KoXmlNS::fo, "keep-together")) {     // was style:break-inside in OOo-1.1, renamed in OASIS
-            if (styleStack.property(KoXmlNS::fo, "keep-together") != "auto")
-                pageBreaking |= KoParagLayout::KeepLinesTogether;
-        }
-        if (styleStack.hasProperty(KoXmlNS::fo, "keep-with-next")) {
-            // OASIS spec says it's "auto"/"always", not a boolean.
-            QString val = styleStack.property(KoXmlNS::fo, "keep-with-next");
-            if (val == "true" || val == "always")
-                pageBreaking |= KoParagLayout::KeepWithNext;
-        }
-    }
-    layout.pageBreaking = pageBreaking;
-#else
-
     // The fo:break-before and fo:break-after attributes insert a page or column break before or after a paragraph.
     if (styleStack.hasProperty(KoXmlNS::fo, "break-before")) {
         if (styleStack.property(KoXmlNS::fo, "break-before") != "auto")
@@ -1295,18 +1302,6 @@ void KoParagraphStyle::loadOdfProperties(KoStyleStack &styleStack)
             setBreakAfter(true);
     }
 
-#endif
-
-#if 0
-    // Paragraph background color -  fo:background-color
-    // The background color for parts of a paragraph that have no text underneath
-    if (styleStack.hasProperty(KoXmlNS::fo, "background-color")) {
-        QString bgColor = styleStack.property(KoXmlNS::fo, "background-color");
-        if (bgColor != "transparent")
-            layout.backgroundColor.setNamedColor(bgColor);
-    }
-#endif
-
     // The fo:background-color attribute specifies the background color of a paragraph.
     if (styleStack.hasProperty(KoXmlNS::fo, "background-color")) {
         const QString bgcolor = styleStack.property(KoXmlNS::fo, "background-color");
@@ -1420,12 +1415,12 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoGenStyles &mainStyles)
         d->charStyle->saveOdf(style);
     }
     if (d->listStyle) {
-        KoGenStyle liststyle(KoGenStyle::StyleList);
+        KoGenStyle liststyle(KoGenStyle::ListStyle);
         d->listStyle->saveOdf(liststyle);
         QString name(QString(QUrl::toPercentEncoding(d->listStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty())
-            name = "L";
-        style.addAttribute("style:list-style-name", mainStyles.lookup(liststyle, name, KoGenStyles::DontForceNumbering));
+            name = 'L';
+        style.addAttribute("style:list-style-name", mainStyles.insert(liststyle, name, KoGenStyles::DontAddNumberToName));
     }
     // only custom style have a displayname. automatic styles don't have a name set.
     if (!d->name.isEmpty() && !style.isDefaultStyle()) {
@@ -1462,15 +1457,8 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoGenStyles &mainStyles)
                     style.addProperty("style:writing-mode", direction, KoGenStyle::ParagraphType);
             }
         } else if (key == QTextFormat::PageBreakPolicy) {
-            QTextFormat::PageBreakFlag pageBreak = static_cast<QTextFormat::PageBreakFlag>(d->stylesPrivate.value(key).toInt());
-            if (pageBreak == QTextFormat::PageBreak_AlwaysBefore)
-                style.addProperty("fo:break-before", "page", KoGenStyle::ParagraphType);
-            else if (pageBreak == QTextFormat::PageBreak_AlwaysAfter)
-                style.addProperty("fo:break-after", "page", KoGenStyle::ParagraphType);
-        } else if (key == KoParagraphStyle::BreakBefore) {
             if (breakBefore())
                 style.addProperty("fo:break-before", "page", KoGenStyle::ParagraphType);
-        } else if (key == KoParagraphStyle::BreakAfter) {
             if (breakAfter())
                 style.addProperty("fo:break-after", "page", KoGenStyle::ParagraphType);
         } else if (key == QTextFormat::BackgroundBrush) {
@@ -1529,6 +1517,8 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoGenStyles &mainStyles)
             style.addPropertyPt("style:tab-stop-distance", tabStopDistance(), KoGenStyle::ParagraphType);
         } else if (key == KoParagraphStyle::MasterPageName) {
             style.addProperty("style:master-page-name", masterPageName());
+        } else if (key == KoParagraphStyle::DefaultOutlineLevel) {
+            style.addProperty("style:default-outline-level", defaultOutlineLevel());
         }
     }
     if (!writtenLineSpacing && normalLineHeight)
