@@ -143,14 +143,14 @@ bool KoSegmentGradient::init()
         data[2] = static_cast<quint8>(leftRed * 255 + 0.5);
         data[1] = static_cast<quint8>(leftGreen * 255 + 0.5);
         data[0] = static_cast<quint8>(leftBlue * 255 + 0.5);
-        data[3] = static_cast<quint8>(leftAlpha * OPACITY_OPAQUE + 0.5);
+        data[3] = static_cast<quint8>(leftAlpha * OPACITY_OPAQUE_U8 + 0.5);
 
         KoColor leftColor(data, rgbColorSpace);
 
         data[2] = static_cast<quint8>(rightRed * 255 + 0.5);
         data[1] = static_cast<quint8>(rightGreen * 255 + 0.5);
         data[0] = static_cast<quint8>(rightBlue * 255 + 0.5);
-        data[3] = static_cast<quint8>(rightAlpha * OPACITY_OPAQUE + 0.5);
+        data[3] = static_cast<quint8>(rightAlpha * OPACITY_OPAQUE_U8 + 0.5);
 
         KoColor rightColor(data, rgbColorSpace);
 
@@ -255,30 +255,27 @@ KoGradientSegment::KoGradientSegment(int interpolationType, int colorInterpolati
 
     if (startOffset < DBL_EPSILON) {
         m_startOffset = 0;
-    } else
-        if (startOffset > 1 - DBL_EPSILON) {
-            m_startOffset = 1;
-        } else {
-            m_startOffset = startOffset;
-        }
+    } else if (startOffset > 1 - DBL_EPSILON) {
+        m_startOffset = 1;
+    } else {
+        m_startOffset = startOffset;
+    }
 
     if (middleOffset < m_startOffset + DBL_EPSILON) {
         m_middleOffset = m_startOffset;
-    } else
-        if (middleOffset > 1 - DBL_EPSILON) {
-            m_middleOffset = 1;
-        } else {
-            m_middleOffset = middleOffset;
-        }
+    } else if (middleOffset > 1 - DBL_EPSILON) {
+        m_middleOffset = 1;
+    } else {
+        m_middleOffset = middleOffset;
+    }
 
     if (endOffset < m_middleOffset + DBL_EPSILON) {
         m_endOffset = m_middleOffset;
-    } else
-        if (endOffset > 1 - DBL_EPSILON) {
-            m_endOffset = 1;
-        } else {
-            m_endOffset = endOffset;
-        }
+    } else if (endOffset > 1 - DBL_EPSILON) {
+        m_endOffset = 1;
+    } else {
+        m_endOffset = endOffset;
+    }
 
     m_length = m_endOffset - m_startOffset;
 
